@@ -4,12 +4,12 @@
 #
 # 1. Update the lists of the ready (about 80%) translations
 # check the situation here: http://l10n.kde.org/stats/gui/trunk-kde4/po/rekonq.po/
-LIST="cs da de en_GB es et fr hu it lt nds pt_BR pt ru sv uk zh_CN"
+LIST="af ar as ast be be@latin bg bn bn_IN br ca ca@valencia crh cs csb cy da de el en_GB eo es et eu fa fi fr fy ga gl gu ha he hi hne hr hsb hu hy ia id is it ja ka kk km kn ko ku lb lt lv mai mk ml mr ms mt nb nds ne nl nn nso oc or pa pl ps pt pt_BR ro ru rw se si sk sl sq sr sr@ijekavian sr@ijekavianlatin sr@latin sv ta te tg th tr uk uz uz@cyrillic vi wa xh zh_CN zh_HK zh_TW"
 
 # 2. run this script. It will create an i18n dir in rekonq sources ($RK_SRCS variable, set it to your source path) 
 # dir with all the listed translations (eg: italian translation = rekonq_it.po file) 
 # plus the CMakeLists.txt file needed to compile them.
-RK_SRCS=/home/adjam/REKO/rekonq-0.5.0
+RK_SRCS=/home/adjam/TMP
 
 # 3. Uncomment the "ADD_SUBDIRECTORY( i18n )" line in main CMakeLists.txt file.
 
@@ -19,12 +19,12 @@ RK_SRCS=/home/adjam/REKO/rekonq-0.5.0
 
 ########################################################################################################
 
-# exit on most errors
-set -e
-
 # current dir
 CWD=$(pwd)
 
+# fix language
+export LANG=en_US.utf8
+
 # create the i18n dir
 cd $RK_SRCS
 mkdir -p i18n
@@ -33,8 +33,37 @@ cd i18n
 # download the po files
 for lang in $LIST
 do
-  wget http://websvn.kde.org/*checkout*/trunk/l10n-kde4/$lang/messages/extragear-network/rekonq.po;
-  mv rekonq.po rekonq_$lang.po;
+    wget http://websvn.kde.org/*checkout*/trunk/l10n-kde4/$lang/messages/extragear-network/rekonq.po
+    
+    if [ -a rekonq.po ]; then
+        mv rekonq.po rekonq_$lang.po
+    
+        # retrieve the statistic string
+        STATS=$(msgfmt --statistic rekonq_$lang.po 2>&1)
+        rm messages.mo
+
+        # grep out translated & untranslated strings number
+        TRANS=$(echo $STATS | awk '{print $1}')
+        TRANS=${TRANS:-0}
+        FUZZ=$(echo $STATS | awk '{print $4}')
+        FUZZ=${FUZZ:-0}
+        UNTR=$(echo $STATS | awk '{print $7}')
+        UNTR=${UNTR:-0}
+
+        # check if perc is more than 80%
+        TOT=$[$TRANS+$FUZZ+$UNTR]
+        if [ $TOT -eq 0 ]; then
+            rm rekonq_$lang.po
+        else
+            PERC=$(echo $TRANS/$TOT | bc -l)
+
+            RESULT=$(echo $PERC '>' .80 | bc -l)
+            if [ $RESULT -eq 0 ]; then 
+                echo removing $lang...
+                rm rekonq_$lang.po
+            fi
+        fi
+    fi
 done
 
 # create the CMakeLists.txt file for the translations
