@@ -67,6 +67,8 @@ void AdBlockManager::loadSettings(bool checkUpdateDate)
     _index = 0;
     _buffer.clear();
 
+    _hostWhiteList.clear();
+    _hostBlackList.clear();
     _whiteList.clear();
     _blackList.clear();
     _hideList.clear();
@@ -133,7 +135,10 @@ void AdBlockManager::loadRules(const QStringList &rules)
         // white rules
         if (stringRule.startsWith(QL1S("@@")))
         {
-            AdBlockRule rule(stringRule.mid(2));
+            const QString filter = stringRule.mid(2);
+            if (_hostWhiteList.tryAddFilter(filter))
+                continue;
+            AdBlockRule rule(filter);
             _whiteList << rule;
             continue;
         }
@@ -145,6 +150,12 @@ void AdBlockManager::loadRules(const QStringList &rules)
             continue;
         }
 
+        // TODO implement domain-specific hiding
+        if (stringRule.contains(QL1S("##")))
+            continue;
+
+        if (_hostBlackList.tryAddFilter(stringRule))
+            continue;
         AdBlockRule rule(stringRule);
         _blackList << rule;
     }
@@ -161,33 +172,50 @@ QNetworkReply *AdBlockManager::block(const QNetworkRequest &request, WebPage *pa
         return 0;
 
     QString urlString = request.url().toString();
+    // We compute a lowercase version of the URL so each rule does not
+    // have to do it.
+    const QString urlStringLowerCase = urlString.toLower();
+    const QString host = request.url().host();
 
     // check white rules before :)
+
+    if (_hostWhiteList.match(host)) {
+        kDebug() << "****ADBLOCK: WHITE RULE (@@) Matched by host matcher: ***********";
+        kDebug() << "UrlString:  " << urlString;
+        return 0;
+    }
+
     foreach(const AdBlockRule &filter, _whiteList)
     {
-        if (filter.match(urlString))
+        if (filter.match(request, urlString, urlStringLowerCase))
         {
             kDebug() << "****ADBLOCK: WHITE RULE (@@) Matched: ***********";
-            kDebug() << "Filter exp: " << filter.pattern();
             kDebug() << "UrlString:  " << urlString;
             return 0;
         }
     }
 
     // then check the black ones :(
+    if (_hostBlackList.match(host)) {
+        kDebug() << "****ADBLOCK: BLACK RULE Matched by host matcher: ***********";
+        kDebug() << "UrlString:  " << urlString;
+        AdBlockNetworkReply *reply = new AdBlockNetworkReply(request, urlString, this);
+        return reply;
+    }
+
     foreach(const AdBlockRule &filter, _blackList)
     {
-        if (filter.match(urlString))
+        if (filter.match(request, urlString, urlStringLowerCase))
         {
             kDebug() << "****ADBLOCK: BLACK RULE Matched: ***********";
-            kDebug() << "Filter exp: " << filter.pattern();
             kDebug() << "UrlString:  " << urlString;
 
             QWebElement document = page->mainFrame()->documentElement();
             QWebElementCollection elements = document.findAll("*");
             foreach(QWebElement el, elements)
             {
-                if (filter.match(el.attribute("src")))
+                const QString srcAttribute = el.attribute("src");
+                if (filter.match(request, srcAttribute, srcAttribute.toLower()))
                 {
                     kDebug() << "MATCHES ATTRIBUTE!!!!!";
                     el.setStyleProperty(QL1S("visibility"), QL1S("hidden"));
@@ -247,6 +275,12 @@ void AdBlockManager::updateNextSubscription()
         KUrl subUrl = KUrl(urlString);
 
         KIO::TransferJob* job = KIO::get(subUrl , KIO::Reload , KIO::HideProgressInfo);
+        job->metaData().insert("ssl_no_client_cert", "TRUE");
+        job->metaData().insert("ssl_no_ui", "TRUE");
+        job->metaData().insert("UseCache", "false");
+        job->metaData().insert("cookies", "none");
+        job->metaData().insert("no-auth", "true");
+
         connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(subscriptionData(KIO::Job*, const QByteArray&)));
         connect(job, SIGNAL(result(KJob*)), this, SLOT(slotResult(KJob*)));
 
