@@ -58,27 +58,27 @@
 // Rekonq Includes
 #include "rekonq_defines.h"
 
-// Qt Includes
-#include <QtCore/QRegExp>
-#include <QtCore/QString>
+#include "adblockruleimpl.h"
+
+#include <QSharedPointer>
 
 // Forward Includes
-class QUrl;
-
+class QNetworkRequest;
+class QString;
 
 class AdBlockRule
 {
 public:
     AdBlockRule(const QString &filter);
 
-    bool match(const QString &encodedUrl) const;
-
-    QString pattern() const;
+    bool match(const QNetworkRequest &request, const QString &encodedUrl, const QString &encodedUrlLowerCase) const
+    {
+        Q_ASSERT(encodedUrl.toLower() == encodedUrlLowerCase);
+        return m_implementation->match(request, encodedUrl, encodedUrlLowerCase);
+    }
 
 private:
-    QString convertPatternToRegExp(const QString &wildcardPattern);
-
-    QRegExp m_regExp;
+    QSharedPointer<AdBlockRuleImpl> m_implementation;
 };
 
 
