@@ -34,37 +34,35 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "mainwindow.h"
-#include "historymanager.h"
-#include "bookmarksmanager.h"
-#include "mainview.h"
-#include "webtab.h"
-#include "urlbar.h"
-#include "sessionmanager.h"
 #include "adblockmanager.h"
-#include "webview.h"
+#include "bookmarkprovider.h"
 #include "filterurljob.h"
+#include "historymanager.h"
+#include "iconmanager.h"
+#include "mainview.h"
+#include "mainwindow.h"
+#include "opensearchmanager.h"
+#include "sessionmanager.h"
+#include "stackedurlbar.h"
 #include "tabbar.h"
+#include "urlbar.h"
+#include "webtab.h"
 
 // KDE Includes
 #include <KCmdLineArgs>
 #include <KStandardDirs>
-#include <kio/job.h>
-#include <kio/jobclasses.h>
-#include <KUriFilter>
 #include <KMessageBox>
-#include <KUrl>
 #include <ThreadWeaver/Weaver>
 
-// Qt Includes
-#include <QtCore/QTimer>
 
-
-QWeakPointer<HistoryManager> Application::s_historyManager;
-QWeakPointer<BookmarkProvider> Application::s_bookmarkProvider;
-QWeakPointer<SessionManager> Application::s_sessionManager;
 QWeakPointer<AdBlockManager> Application::s_adblockManager;
+QWeakPointer<BookmarkProvider> Application::s_bookmarkProvider;
+QWeakPointer<HistoryManager> Application::s_historyManager;
+QWeakPointer<IconManager> Application::s_iconManager;
+QWeakPointer<OpenSearchManager> Application::s_opensearchManager;
+QWeakPointer<SessionManager> Application::s_sessionManager;
 
+using namespace ThreadWeaver;
 
 Application::Application()
         : KUniqueApplication()
@@ -98,6 +96,9 @@ Application::~Application()
 
     delete s_adblockManager.data();
     s_adblockManager.clear();
+
+    delete s_opensearchManager.data();
+    s_opensearchManager.clear();
 }
 
 
@@ -107,7 +108,7 @@ int Application::newInstance()
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
 
     // not that easy, indeed
-    // We have to consider 3 variables here: 
+    // We have to consider 3 variables here:
     // 1) Is first load?
     // 2) Are there arguments?
     // 3) Is rekonq recovering from crash?
@@ -115,13 +116,13 @@ int Application::newInstance()
     bool isFirstLoad = m_mainWindows.isEmpty();
     bool areThereArguments = (args->count() > 0);
     bool isRekonqCrashed = (ReKonfig::recoverOnCrash() == 1);
-    
+
     kDebug() << "is first load? " << isFirstLoad;
     kDebug() << "are there arguments? " << areThereArguments;
     kDebug() << "is rekonq crashed? " << isRekonqCrashed;
-    
+
     int exitValue = 1 * isFirstLoad + 2 * areThereArguments + 4 * isRekonqCrashed;
-    
+
     if(isRekonqCrashed)
     {
         if( isFirstLoad  && sessionManager()->restoreSession() )
@@ -139,7 +140,7 @@ int Application::newInstance()
                 loadUrl( KUrl("about:blank"), Rekonq::NewFocusedTab);
         }
     }
-    
+
     if(areThereArguments)
     {
         KUrl::List urlList;
@@ -151,34 +152,23 @@ int Application::newInstance()
             else
                 urlList += KUrl( args->arg(i) ); // "rekonq kde.org" || "rekonq kde:kdialog" case
         }
-        
+
         if (isFirstLoad)
         {
             // No windows in the current desktop? No windows at all?
             // Create a new one and load there sites...
             loadUrl(urlList.at(0), Rekonq::CurrentTab);
-            for (int i = 1; i < urlList.count(); ++i)
-                loadUrl( urlList.at(i), Rekonq::NewTab);
         }
         else
         {
-            // are there any windows there? use it
-            int index = m_mainWindows.size();
-            if (index > 0)
-            {
-                MainWindow *m = m_mainWindows.at(index - 1).data();
-                if(m->isMinimized())
-                    m->showNormal();
-                if( !m->isActiveWindow() )
-                {
-                    m->activateWindow();
-                    m->raise();
-                }
-
-                Q_FOREACH(const KUrl &u, urlList)
-                    loadUrl(u, Rekonq::NewFocusedTab);
-            }
-        }    
+            if(ReKonfig::openTabNoWindow())
+                loadUrl(urlList.at(0), Rekonq::NewTab);
+            else
+                loadUrl(urlList.at(0), Rekonq::NewWindow);
+        }
+        
+        for (int i = 1; i < urlList.count(); ++i)
+            loadUrl( urlList.at(i), Rekonq::NewTab);
     }
     else
     {
@@ -218,15 +208,15 @@ int Application::newInstance()
                 break;
            }
 
-        }    
+        }
     }
-    
-    
+
     if(isFirstLoad)
     {
-        QTimer::singleShot(0, this, SLOT(postLaunch()));
+        // give me some time to do the other things..
+        QTimer::singleShot(100, this, SLOT(postLaunch()));
     }
-    
+
     return exitValue;
 }
 
@@ -244,10 +234,6 @@ void Application::postLaunch()
 
     setWindowIcon(KIcon("rekonq"));
 
-    // set Icon Database Path to store "favicons" associated with web sites
-    QString directory = KStandardDirs::locateLocal("cache" , "" , true);
-    QWebSettings::setIconDatabasePath(directory);
-
     Application::historyManager();
     Application::sessionManager();
 
@@ -314,33 +300,24 @@ SessionManager *Application::sessionManager()
 }
 
 
-KIcon Application::icon(const KUrl &url)
+OpenSearchManager *Application::opensearchManager()
 {
-    // avoid infinite loop at startup
-    if (Application::instance()->mainWindowList().isEmpty())
-        return KIcon("text-html");
-
-    // first things first..
-    if (url.isEmpty())
-        return KIcon("text-html");
-
-    // rekonq icons..
-    if (url == KUrl("about:closedTabs"))
-        return KIcon("tab-close");
-    if (url == KUrl("about:history"))
-        return KIcon("view-history");
-    if (url == KUrl("about:bookmarks"))
-        return KIcon("bookmarks");
-    if (url == KUrl("about:favorites"))
-        return KIcon("emblem-favorite");
-
-    KIcon icon = KIcon(QWebSettings::iconForUrl(url));
-    if (icon.isNull())
+    if (s_opensearchManager.isNull())
     {
-        kDebug() << "null icon";
-        icon = KIcon("text-html");
+        s_opensearchManager = new OpenSearchManager(instance());
+        s_opensearchManager.data()->setSearchProvider("google"); //TODO: use other suggestion engines
     }
-    return icon;
+    return s_opensearchManager.data();
+}
+
+
+IconManager *Application::iconManager()
+{
+    if (s_iconManager.isNull())
+    {
+        s_iconManager = new IconManager(instance());
+    }
+    return s_iconManager.data();
 }
 
 
@@ -391,7 +368,7 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
     Q_ASSERT( tabIndex != -1 );
     UrlBar *barForTab = qobject_cast<UrlBar *>(w->mainView()->widgetBar()->widget(tabIndex));
     barForTab->setQUrl(url);
-    
+
     WebView *view = tab->view();
 
     if (view)
@@ -444,19 +421,13 @@ void Application::loadResolvedUrl(ThreadWeaver::Job *job)
     KUrl url = threadedJob->url();
     WebView *view = threadedJob->view();
 
-    // Bye and thanks :)
-    delete threadedJob;
-
     if (view)
     {
         view->load(url);
-
-        // we are sure of the url now, let's add it to history
-        // anyway we store here just http sites because local and ftp ones are
-        // added trough the protocol handler and the other are ignored
-        if (url.protocol() == QL1S("http") || url.protocol() == QL1S("https"))
-            historyManager()->addHistoryEntry(url.prettyUrl());
     }
+
+    // Bye and thanks :)
+    delete threadedJob;
 }
 
 
@@ -475,16 +446,16 @@ void Application::updateConfiguration()
     {
         MainView *mv = w.data()->mainView();
         mv->updateTabBar();
-    
+
         if (b)
             mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
         else
             mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectRightTab);
     }
 
-    QWebSettings *defaultSettings = QWebSettings::globalSettings();    
-    
-    // =========== Fonts ==============    
+    QWebSettings *defaultSettings = QWebSettings::globalSettings();
+
+    // =========== Fonts ==============
     defaultSettings->setFontFamily(QWebSettings::StandardFont, ReKonfig::standardFontFamily() );
     defaultSettings->setFontFamily(QWebSettings::FixedFont, ReKonfig::fixedFontFamily() );
     defaultSettings->setFontFamily(QWebSettings::SerifFont, ReKonfig::serifFontFamily() );
@@ -492,15 +463,15 @@ void Application::updateConfiguration()
     defaultSettings->setFontFamily(QWebSettings::CursiveFont, ReKonfig::cursiveFontFamily());
     defaultSettings->setFontFamily(QWebSettings::FantasyFont, ReKonfig::fantasyFontFamily());
 
-    // compute font size 
+    // compute font size
     // (I have to admit I know nothing about these DPI questions..: copied from kwebkitpart, as someone suggested)
     // font size in pixels =  font size in inches × screen dpi
-    int defaultFontSize = ReKonfig::defaultFontSize();    
+    int defaultFontSize = ReKonfig::defaultFontSize();
     int minimumFontSize = ReKonfig::minFontSize();
-    
+
     int logDpiY = mainWindow()->currentTab()->view()->logicalDpiY();
     kDebug() << "Logical Dot per Inch Y: " << logDpiY;
-    
+
     float toPix = (logDpiY < 96.0)
         ? 96.0/72.0
         : logDpiY/72.0 ;
@@ -508,7 +479,7 @@ void Application::updateConfiguration()
     defaultSettings->setFontSize(QWebSettings::DefaultFontSize, qRound(defaultFontSize * toPix) );
     defaultSettings->setFontSize(QWebSettings::MinimumFontSize, qRound(minimumFontSize * toPix) );
 
-    
+
     // ================ WebKit ============================
     defaultSettings->setAttribute(QWebSettings::AutoLoadImages, ReKonfig::autoLoadImages());
     defaultSettings->setAttribute(QWebSettings::DnsPrefetchEnabled, ReKonfig::dnsPrefetch());
@@ -548,3 +519,59 @@ void Application::updateConfiguration()
 
     defaultSettings = 0;
 }
+
+
+void Application::addDownload(const QString &srcUrl, const QString &destUrl)
+{
+    QWebSettings *globalSettings = QWebSettings::globalSettings();
+    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
+        return;
+    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
+    QFile downloadFile(downloadFilePath);
+    if (!downloadFile.open(QFile::WriteOnly | QFile::Append))
+    {
+        kDebug() << "Unable to open download file (WRITE mode)..";
+        return;
+    }
+    QDataStream out(&downloadFile);
+    out << srcUrl;
+    out << destUrl;
+    out << QDateTime::currentDateTime();
+    downloadFile.close();
+}
+
+
+DownloadList Application::downloads()
+{
+    DownloadList list;
+
+    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
+    QFile downloadFile(downloadFilePath);
+    if (!downloadFile.open(QFile::ReadOnly))
+    {
+        kDebug() << "Unable to open download file (READ mode)..";
+        return list;
+    }
+
+    QDataStream in(&downloadFile);
+    while (!in.atEnd())
+    {
+        QString srcUrl;
+        in >> srcUrl;
+        QString destUrl;
+        in >> destUrl;
+        QDateTime dt;
+        in >> dt;
+        DownloadItem item(srcUrl, destUrl, dt);
+        list << item;
+    }
+    return list;
+}
+
+
+bool Application::clearDownloadsHistory()
+{
+    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
+    QFile downloadFile(downloadFilePath);
+    return downloadFile.remove();
+}
