@@ -35,25 +35,48 @@
 
 // KDE Includes
 #include <KUniqueApplication>
-#include <KIcon>
-#include <ThreadWeaver/Job>
 
 // Qt Includes
-#include <QWeakPointer>
-#include <QList>
+#include <QtCore/QDateTime>
+#include <QtCore/QWeakPointer>
 
 // Forward Declarations
-class KIcon;
-class KUrl;
-class BookmarkProvider;
-class HistoryManager;
-class MainWindow;
-class SessionManager;
 class AdBlockManager;
-class WebView;
+class BookmarkProvider;
+class DownloadItem;
+class HistoryManager;
+class IconManager;
+class MainWindow;
+class OpenSearchManager;
+class SessionManager;
+
+namespace ThreadWeaver {class Job;}
 
 
 typedef QList< QWeakPointer<MainWindow> > MainWindowList;
+typedef QList<DownloadItem> DownloadList;
+
+
+class DownloadItem
+{
+public:
+    DownloadItem() {}
+    explicit DownloadItem(const QString &srcUrl,
+                          const QString &destUrl,
+                          const QDateTime &d
+                         )
+            : srcUrlString(srcUrl)
+            , destUrlString(destUrl)
+            , dateTime(d)
+    {}
+
+    QString srcUrlString;
+    QString destUrlString;
+    QDateTime dateTime;
+};
+
+
+// ---------------------------------------------------------------------------------------------------------------
 
 
 /**
@@ -73,12 +96,17 @@ public:
     MainWindow *newMainWindow(bool withTab = true);
     MainWindowList mainWindowList();
 
-    static KIcon icon(const KUrl &url);
-
     static HistoryManager *historyManager();
     static BookmarkProvider *bookmarkProvider();
     static SessionManager *sessionManager();
     static AdBlockManager *adblockManager();
+    static OpenSearchManager *opensearchManager();
+    static IconManager *iconManager();
+
+    // DOWNLOADS MANAGEMENT METHODS
+    void addDownload(const QString &srcUrl, const QString &destUrl);
+    DownloadList downloads();
+    bool clearDownloadsHistory();
 
 public slots:
     /**
@@ -111,6 +139,8 @@ private:
     static QWeakPointer<BookmarkProvider> s_bookmarkProvider;
     static QWeakPointer<SessionManager> s_sessionManager;
     static QWeakPointer<AdBlockManager> s_adblockManager;
+    static QWeakPointer<OpenSearchManager> s_opensearchManager;
+    static QWeakPointer<IconManager> s_iconManager;
 
     MainWindowList m_mainWindows;
 };
