@@ -34,53 +34,48 @@
 #include "rekonq_defines.h"
 
 // Local Includes
-#include "application.h"
-#include "paneltreeview.h"
-
-// Qt Includes
-#include <QDockWidget>
-
-// KDE Includes
-#include <KBookmark>
+#include "urlpanel.h"
 
 // Forward Declarations
-class KUrl;
+class BookmarksTreeModel;
+
+class KBookmark;
 class QModelIndex;
 
 
-class REKONQ_TESTS_EXPORT BookmarksPanel : public QDockWidget
+class REKONQ_TESTS_EXPORT BookmarksPanel : public UrlPanel
 {
     Q_OBJECT
 
 public:
     explicit BookmarksPanel(const QString &title, QWidget *parent = 0, Qt::WindowFlags flags = 0);
-    ~BookmarksPanel();
+    virtual ~BookmarksPanel();
 
-signals:
-    void openUrl(const KUrl &, const Rekonq::OpenType &);
-    void itemHovered(const QString &);
-    void saveOnlyRequested();
+public Q_SLOTS:
+    void startLoadFoldedState();
 
-public slots:
-    void showing(bool);
-    
-private slots:
+private Q_SLOTS:
     void contextMenu(const QPoint &pos);
+    virtual void contextMenuItem(const QPoint &pos);
+    virtual void contextMenuGroup(const QPoint &pos);
+    virtual void contextMenuEmpty(const QPoint &pos);
 
     void deleteBookmark();
     void onCollapse(const QModelIndex &index);
     void onExpand(const QModelIndex &index);
     void loadFoldedState(const QModelIndex &root);
-    void loadFoldedState();
+
+Q_SIGNALS:
+    void expansionChanged();
 
 private:
-    void setup();
+    virtual void setup();
     KBookmark bookmarkForIndex(const QModelIndex &index);
 
-    PanelTreeView *m_treeView;
-    bool m_loadingState;
-    
-    bool _loaded;
+    virtual QAbstractItemModel* getModel();
+
+    BookmarksTreeModel *_bkTreeModel;
+    bool _loadingState;
 };
 
 #endif // BOOKMARKSPANEL_H
