@@ -31,14 +31,16 @@
 
 // Local Includes
 #include "application.h"
-#include "bookmarksmanager.h"
+#include "bookmarkprovider.h"
+#include "iconmanager.h"
+
+// KDE Includes
+#include <KBookmarkManager>
+#include <KLocalizedString>
+#include <KIcon>
 
 // Qt Includes
-#include <QMimeData>
-
-// KDE includes
-#include <KBookmarkGroup>
-#include <KLocalizedString>
+#include <QtCore/QMimeData>
 
 
 BtmItem::BtmItem(const KBookmark &bm)
@@ -61,28 +63,37 @@ QVariant BtmItem::data(int role) const
 
     if (role == Qt::DisplayRole)
         return m_kbm.text();
+
     if (role == Qt::DecorationRole)
-        return KIcon(m_kbm.icon());
+    {
+        // NOTE
+        // this should be:
+        // return KIcon(m_kbm.icon());
+        // but I cannot let it work :(
+        // I really cannot understand how let this work properly...
+        if (m_kbm.isGroup() || m_kbm.isSeparator())
+            return KIcon(m_kbm.icon());
+        else
+            return Application::iconManager()->iconForUrl(KUrl(m_kbm.url()));
+    }
+
     if (role == Qt::UserRole)
         return m_kbm.url();
+
     if (role == Qt::ToolTipRole)
     {
-        QString tooltip = "";
-
-        if (!m_kbm.text().isEmpty())
-        {
-            tooltip += m_kbm.text();
-        }
+        QString tooltip = m_kbm.fullText();
         if (m_kbm.isGroup())
-        {
             tooltip += i18ncp("%1=Number of items in bookmark folder", " (1 item)", " (%1 items)", childCount());
-        }
-        if (!m_kbm.url().url().isEmpty())
+
+        QString url = m_kbm.url().url();
+        if (!url.isEmpty())
         {
             if (!tooltip.isEmpty())
                 tooltip += '\n';
-            tooltip += m_kbm.url().url();
+            tooltip += url;
         }
+
         return tooltip;
     }
 
@@ -140,6 +151,7 @@ KBookmark BtmItem::getBkm() const
     return m_kbm;
 }
 
+
 // -------------------------------------------------------------------------------------
 
 
@@ -148,9 +160,7 @@ BookmarksTreeModel::BookmarksTreeModel(QObject *parent)
         , m_root(0)
 {
     resetModel();
-    connect(this, SIGNAL(bookmarksUpdated()), parent, SLOT(loadFoldedState()));
-    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(QString, QString)), this, SLOT(bookmarksChanged()));
-    connect(parent, SIGNAL(saveOnlyRequested()), this, SLOT(saveOnly()));
+    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(const QString &, const QString &)), this, SLOT(bookmarksChanged(const QString &)));
 }
 
 
@@ -169,33 +179,19 @@ int BookmarksTreeModel::rowCount(const QModelIndex &parent) const
     }
     else
     {
-        parentItem = static_cast< BtmItem* >(parent.internalPointer());
+        parentItem = static_cast<BtmItem*>(parent.internalPointer());
     }
 
     return parentItem->childCount();
 }
 
 
-int BookmarksTreeModel::columnCount(const QModelIndex &parent) const
+int BookmarksTreeModel::columnCount(const QModelIndex& /*parent*/) const
 {
-    Q_UNUSED(parent)
-    // name
     return 1;
 }
 
 
-QVariant BookmarksTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
-{
-    if (orientation == Qt::Horizontal
-            && role == Qt::DisplayRole
-            && section == 0
-       )
-        return i18n("Bookmark");
-
-    return QVariant();
-}
-
-
 Qt::ItemFlags BookmarksTreeModel::flags(const QModelIndex &index) const
 {
     Qt::ItemFlags flags = QAbstractItemModel::flags(index);
@@ -215,26 +211,18 @@ Qt::ItemFlags BookmarksTreeModel::flags(const QModelIndex &index) const
 QModelIndex BookmarksTreeModel::index(int row, int column, const QModelIndex &parent) const
 {
     if (!hasIndex(row, column, parent))
-    {
         return QModelIndex();
-    }
 
     BtmItem *parentItem;
 
     if (!parent.isValid())
-    {
         parentItem = m_root;
-    }
     else
-    {
-        parentItem = static_cast< BtmItem* >(parent.internalPointer());
-    }
+        parentItem = static_cast<BtmItem*>(parent.internalPointer());
 
     BtmItem *childItem = parentItem->child(row);
     if (childItem)
-    {
         return createIndex(row, column, childItem);
-    }
 
     return QModelIndex();
 }
@@ -243,17 +231,13 @@ QModelIndex BookmarksTreeModel::index(int row, int column, const QModelIndex &pa
 QModelIndex BookmarksTreeModel::parent(const QModelIndex &index) const
 {
     if (!index.isValid())
-    {
         return QModelIndex();
-    }
 
-    BtmItem *childItem = static_cast< BtmItem* >(index.internalPointer());
+    BtmItem *childItem = static_cast<BtmItem*>(index.internalPointer());
     BtmItem *parentItem = childItem->parent();
 
     if (parentItem == m_root)
-    {
         return QModelIndex();
-    }
 
     return createIndex(parentItem->row(), 0, parentItem);
 }
@@ -262,11 +246,9 @@ QModelIndex BookmarksTreeModel::parent(const QModelIndex &index) const
 QVariant BookmarksTreeModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid())
-    {
         return QVariant();
-    }
 
-    BtmItem *node = static_cast< BtmItem* >(index.internalPointer());
+    BtmItem *node = static_cast<BtmItem*>(index.internalPointer());
     if (node && node == m_root)
     {
         if (role == Qt::DisplayRole)
@@ -284,9 +266,93 @@ QVariant BookmarksTreeModel::data(const QModelIndex &index, int role) const
 }
 
 
-void BookmarksTreeModel::bookmarksChanged()
+QStringList BookmarksTreeModel::mimeTypes() const
 {
-    resetModel();
+    return QStringList("application/rekonq-bookmark");
+}
+
+
+bool BookmarksTreeModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent)
+{
+    if (action != Qt::MoveAction || !data->hasFormat("application/rekonq-bookmark"))
+        return false;
+
+    QByteArray addresses = data->data("application/rekonq-bookmark");
+    KBookmark bookmark = Application::bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+
+    KBookmarkGroup root;
+    if (parent.isValid())
+        root = bookmarkForIndex(parent).toGroup();
+    else
+        root = Application::bookmarkProvider()->rootGroup();
+
+    QModelIndex destIndex = index(row, column, parent);
+
+    if (destIndex.isValid() && row != -1)
+    {
+        root.moveBookmark(bookmark, root.previous(bookmarkForIndex(destIndex)));
+    }
+    else
+    {
+        root.deleteBookmark(bookmark);
+        root.addBookmark(bookmark);
+    }
+
+    Application::bookmarkProvider()->bookmarkManager()->emitChanged();
+
+    return true;
+}
+
+
+Qt::DropActions BookmarksTreeModel::supportedDropActions() const
+{
+    return Qt::MoveAction;
+}
+
+
+QMimeData* BookmarksTreeModel::mimeData(const QModelIndexList &indexes) const
+{
+    QMimeData *mimeData = new QMimeData;
+
+    QByteArray address = bookmarkForIndex(indexes.first()).address().toLatin1();
+    mimeData->setData("application/rekonq-bookmark", address);
+    bookmarkForIndex(indexes.first()).populateMimeData(mimeData);
+
+    return mimeData;
+}
+
+
+void BookmarksTreeModel::bookmarksChanged(const QString &groupAddress)
+{
+    if (groupAddress.isEmpty())
+    {
+        resetModel();
+    }
+    else
+    {
+        beginResetModel();
+        BtmItem *node = m_root;
+        QModelIndex nodeIndex;
+
+        QStringList indexChain( groupAddress.split('/', QString::SkipEmptyParts) );
+        bool ok;
+        int i;
+        foreach (const QString &sIndex, indexChain)
+        {
+            i = sIndex.toInt( &ok );
+            if( !ok )
+                break;
+
+            if( i < 0 || i >= node->childCount() )
+                break;
+
+            node = node->child( i );
+            nodeIndex = index( i, 0, nodeIndex );
+        }
+        populate(node, Application::bookmarkProvider()->bookmarkManager()->findByAddress(groupAddress).toGroup());
+        endResetModel();
+    }
+
     emit bookmarksUpdated();
 }
 
@@ -299,14 +365,11 @@ void BookmarksTreeModel::resetModel()
 
 void BookmarksTreeModel::setRoot(KBookmarkGroup bmg)
 {
+    beginResetModel();
     delete m_root;
     m_root = new BtmItem(KBookmark());
-
-    if (bmg.isNull())
-        return;
-
     populate(m_root, bmg);
-    reset();
+    endResetModel();
 }
 
 
@@ -334,94 +397,3 @@ KBookmark BookmarksTreeModel::bookmarkForIndex(const QModelIndex &index) const
 {
     return static_cast<BtmItem*>(index.internalPointer())->getBkm();
 }
-
-
-void BookmarksTreeModel::saveOnly()
-{
-    disconnect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(QString, QString)), this, SLOT(bookmarksChanged()));
-    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(QString, QString)), this, SLOT(reconnectManager()));
-    Application::bookmarkProvider()->bookmarkManager()->emitChanged();
-}
-
-
-void BookmarksTreeModel::reconnectManager()
-{
-    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(QString, QString)), this, SLOT(bookmarksChanged()));
-}
-
-
-Qt::DropActions BookmarksTreeModel::supportedDropActions() const
-{
-    return Qt::MoveAction;
-}
-
-
-QStringList BookmarksTreeModel::mimeTypes() const
-{
-    return KBookmark::List::mimeDataTypes();
-}
-
-
-QMimeData* BookmarksTreeModel::mimeData(const QModelIndexList & indexes) const
-{
-    QMimeData *mimeData = new QMimeData;
-
-    QByteArray addresse = bookmarkForIndex(indexes.first()).address().toLatin1();
-    mimeData->setData("application/rekonq-bookmark", addresse);
-    bookmarkForIndex(indexes.first()).populateMimeData(mimeData);
-
-    return mimeData;
-}
-
-
-bool BookmarksTreeModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex & parent)
-{
-    if (action == Qt::MoveAction)
-    {
-        if (data->hasFormat("application/rekonq-bookmark"))
-        {
-            QByteArray addresses = data->data("application/rekonq-bookmark");
-            KBookmark bookmark = Application::bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
-
-            QModelIndex destIndex = index(row, column, parent);
-
-            KBookmark dropDestBookmark;
-            if (destIndex.isValid())
-                dropDestBookmark = bookmarkForIndex(destIndex);
-
-            KBookmarkGroup root = Application::bookmarkProvider()->rootGroup();
-            if (parent.isValid())
-                root = bookmarkForIndex(parent).toGroup();
-
-            if (!destIndex.isValid())
-            {
-                if (!parent.isValid()) // Drop into a blank area
-                {
-                    Application::bookmarkProvider()->rootGroup().deleteBookmark(bookmark);
-                    Application::bookmarkProvider()->rootGroup().addBookmark(bookmark);
-                }
-                else // Drop at the last item of the group or directly on the main item of the group
-                {
-                    root.deleteBookmark(bookmark);
-                    root.addBookmark(bookmark);
-                }
-            }
-
-            else
-            {
-                if (row == -1)
-                {
-                    root.deleteBookmark(bookmark);
-                    root.addBookmark(bookmark);
-                }
-                else // A classic drop
-                {
-                    root.moveBookmark(bookmark, root.previous(dropDestBookmark));
-                }
-            }
-
-            Application::bookmarkProvider()->bookmarkManager()->emitChanged(root);
-        }
-    }
-    return true;
-}
