@@ -36,13 +36,15 @@
 #include <KBookmark>
 
 // Qt Includes
-#include <QAbstractItemModel>
+#include <QtCore/QAbstractItemModel>
+
 
 class BtmItem
 {
 public:
     BtmItem(const KBookmark &bm);
     ~BtmItem();
+
     QVariant data(int role = Qt::DisplayRole) const;
     int row() const;
     int childCount() const;
@@ -59,45 +61,56 @@ private:
 };
 
 
+// -------------------------------------------------------------------------------------------------
+
+
 class REKONQ_TESTS_EXPORT BookmarksTreeModel : public QAbstractItemModel
 {
     Q_OBJECT
-    Q_DISABLE_COPY(BookmarksTreeModel)
 
 public:
     explicit BookmarksTreeModel(QObject *parent = 0);
-    ~BookmarksTreeModel();
+    virtual ~BookmarksTreeModel();
 
+    /**
+     * @return number of rows under the given parent.
+     */
     virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    /**
+     * @return number of columns (always 1).
+     */
     virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
 
-    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
     virtual Qt::ItemFlags flags(const QModelIndex &index) const;
 
+    /**
+     * @return index in the model specified by the given row, column and parent.
+     */
     virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
+    /**
+     * @return parent of the given index.
+     */
     virtual QModelIndex parent(const QModelIndex &index) const;
     virtual QVariant data(const QModelIndex &index, int role) const;
 
     virtual QStringList mimeTypes() const;
-    virtual bool dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent);
+    virtual bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
     virtual Qt::DropActions supportedDropActions() const;
-    virtual QMimeData *mimeData(const QModelIndexList & indexes) const;
+    virtual QMimeData *mimeData(const QModelIndexList &indexes) const;
 
-private slots:
-    void bookmarksChanged();
-    void saveOnly();
-    void reconnectManager();
+private Q_SLOTS:
+    void bookmarksChanged(const QString &groupAddress);
 
-signals:
+Q_SIGNALS:
     void bookmarksUpdated();
 
 private:
-    BtmItem *m_root;
-
     void resetModel();
     void setRoot(KBookmarkGroup bmg);
     void populate(BtmItem *node, KBookmarkGroup bmg);
     KBookmark bookmarkForIndex(const QModelIndex &index) const;
+
+    BtmItem *m_root;
 };
 
 #endif // BOOKMARKSTREEMODEL_H
