@@ -33,32 +33,30 @@
 #include "mainwindow.h"
 
 // KDE Includes
-#include <KLineEdit>
-#include <KIcon>
-#include <KPushButton>
-#include <klocalizedstring.h>
 #include <KApplication>
+#include <KIcon>
+#include <KLineEdit>
+#include <KLocalizedString>
+#include <KPushButton>
 
 // Qt Includes
-#include <QtGui/QWidget>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QToolButton>
-#include <QtGui/QLabel>
-#include <QtGui/QColor>
-#include <QtCore/QString>
 #include <QtCore/QTimer>
 
+#include <QtGui/QCheckBox>
+#include <QtGui/QHBoxLayout>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QToolButton>
 
-FindBar::FindBar(QWidget *parent)
-        : QWidget(parent)
+
+FindBar::FindBar(MainWindow *window)
+        : QWidget(window)
+        , m_mainWindow(window)
         , m_lineEdit(new KLineEdit(this))
         , m_hideTimer(new QTimer(this))
         , m_matchCase(new QCheckBox(i18n("&Match case"), this))
         , m_highlightAll(new QCheckBox(i18n("&Highlight all"), this))
 {
-    // mainwindow pointer
-    MainWindow *window = qobject_cast<MainWindow *>(parent);
-
     QHBoxLayout *layout = new QHBoxLayout;
 
     // cosmetic
@@ -69,12 +67,12 @@ FindBar::FindBar(QWidget *parent)
     hideButton->setAutoRaise(true);
     hideButton->setIcon(KIcon("dialog-close"));
     connect(hideButton, SIGNAL(clicked()), this, SLOT(hide()));
-    connect(hideButton, SIGNAL(clicked()), window, SLOT(highlightAll()));
     layout->addWidget(hideButton);
     layout->setAlignment(hideButton, Qt::AlignLeft | Qt::AlignTop);
 
     // hide timer
     connect(m_hideTimer, SIGNAL(timeout()), this, SLOT(hide()));
+    m_hideTimer->setSingleShot(true);
 
     // label
     QLabel *label = new QLabel(i18n("Find:"));
@@ -84,7 +82,6 @@ FindBar::FindBar(QWidget *parent)
     setFocusProxy(m_lineEdit);
     m_lineEdit->setMaximumWidth(250);
     connect(m_lineEdit, SIGNAL(textChanged(const QString &)), window, SLOT(find(const QString &)));
-    connect(m_lineEdit, SIGNAL(returnPressed()), window, SLOT(findNext()));
     layout->addWidget(m_lineEdit);
 
     // buttons
@@ -104,7 +101,7 @@ FindBar::FindBar(QWidget *parent)
     // Hightlight All. On by default
     m_highlightAll->setCheckState(Qt::Checked);
     m_highlightAll->setTristate(false);
-    connect(m_highlightAll, SIGNAL(toggled(bool)), window, SLOT(highlightAll()));
+    connect(m_highlightAll, SIGNAL(toggled(bool)), window, SLOT(updateHighlight()));
     layout->addWidget(m_highlightAll);
 
     // stretching widget on the left
@@ -126,6 +123,23 @@ FindBar::~FindBar()
 }
 
 
+void FindBar::keyPressEvent(QKeyEvent *event)
+{
+    if (event->key() == Qt::Key_Return)
+    {
+        if (event->modifiers() == Qt::ShiftModifier)
+        {
+            m_mainWindow->findPrevious();
+        }
+        else
+        {
+            m_mainWindow->findNext();
+        }
+    }
+    QWidget::keyPressEvent(event);
+}
+
+
 bool FindBar::matchCase() const
 {
     return m_matchCase->isChecked();
@@ -138,23 +152,40 @@ bool FindBar::highlightAllState() const
 }
 
 
-void FindBar::show()
+void FindBar::setVisible(bool visible)
 {
-    // show findbar if not visible
-    if (isHidden())
+    QWidget::setVisible(visible);
+
+    if (visible)
     {
-        QWidget::show();
-        emit searchString(m_lineEdit->text());
+        const QString selectedText = m_mainWindow->selectedText();
+        if (!hasFocus() && !selectedText.isEmpty())
+        {
+            const QString previousText = m_lineEdit->text();
+             m_lineEdit->setText(selectedText);
+
+             if (m_lineEdit->text() != previousText)
+                 m_mainWindow->findPrevious();
+             else
+                 m_mainWindow->updateHighlight();;
+        }
+        else if (selectedText.isEmpty())
+        {
+            emit searchString(m_lineEdit->text());
+        }
+
+        m_hideTimer->start(60000);
+
+        m_lineEdit->setFocus();
+        m_lineEdit->selectAll();
+    }
+    else
+    {
+        m_mainWindow->updateHighlight();;
+        m_hideTimer->stop();
     }
-
-    m_hideTimer->start(60000);
-
-    // set focus to findbar if user select showFindBar shortcut
-    m_lineEdit->setFocus();
-    m_lineEdit->selectAll();
 }
 
-
 void FindBar::notifyMatch(bool match)
 {
     QPalette p = m_lineEdit->palette();
@@ -177,11 +208,3 @@ void FindBar::notifyMatch(bool match)
     m_lineEdit->setPalette(p);
     m_hideTimer->start(60000);
 }
-
-
-void FindBar::hide()
-{
-    m_hideTimer->stop();
-    QWidget::hide();
-    emit(searchString(m_lineEdit->text()));
-}
