@@ -32,15 +32,16 @@
 // Rekonq Includes
 #include "rekonq_defines.h"
 
-// KDE Includes
-#include <KLineEdit>
-
 // Qt Includes
 #include <QtGui/QWidget>
-#include <QtGui/QCheckBox>
 
 // Forward Declarations
+class MainWindow;
+
+class KLineEdit;
+
 class QString;
+class QCheckBox;
 
 
 class REKONQ_TESTS_EXPORT FindBar : public QWidget
@@ -48,21 +49,23 @@ class REKONQ_TESTS_EXPORT FindBar : public QWidget
     Q_OBJECT
 
 public:
-    FindBar(QWidget *parent);
+    FindBar(MainWindow *parent);
     ~FindBar();
 
     bool matchCase() const;
     void notifyMatch(bool match);
     bool highlightAllState() const;
 
-public slots:
-    void show();
-    void hide();
+    void setVisible(bool visible);
 
-signals:
+protected:
+    void keyPressEvent(QKeyEvent *event);
+
+Q_SIGNALS:
     void searchString(const QString &);
 
 private:
+    MainWindow *m_mainWindow;
     KLineEdit *m_lineEdit;
     QTimer *m_hideTimer;
     QCheckBox *m_matchCase;
