@@ -65,25 +65,18 @@ static const unsigned int HISTORY_VERSION = 23;
 HistoryManager::HistoryManager(QObject *parent)
         : QWebHistoryInterface(parent)
         , m_saveTimer(new AutoSaver(this))
-        , m_historyLimit(30)
-        , m_historyModel(0)
-        , m_historyFilterModel(0)
+        , m_historyLimit(0)
         , m_historyTreeModel(0)
-        , m_completion(new AwesomeUrlCompletion)
 {
     kDebug() << "Loading HistoryManager...";
-    // take care of the completion object
-    m_completion->setOrder(KCompletion::Weighted);
 
-    m_expiredTimer.setSingleShot(true);
-    connect(&m_expiredTimer, SIGNAL(timeout()), this, SLOT(checkForExpired()));
     connect(this, SIGNAL(entryAdded(const HistoryItem &)), m_saveTimer, SLOT(changeOccurred()));
     connect(this, SIGNAL(entryRemoved(const HistoryItem &)), m_saveTimer, SLOT(changeOccurred()));
 
     load();
 
-    m_historyModel = new HistoryModel(this, this);
-    m_historyFilterModel = new HistoryFilterModel(m_historyModel, this);
+    HistoryModel *historyModel = new HistoryModel(this, this);
+    m_historyFilterModel = new HistoryFilterModel(historyModel, this);
     m_historyTreeModel = new HistoryTreeModel(m_historyFilterModel, this);
 
     // QWebHistoryInterface will delete the history manager
@@ -95,19 +88,11 @@ HistoryManager::HistoryManager(QObject *parent)
 HistoryManager::~HistoryManager()
 {
     m_saveTimer->saveIfNeccessary();
-    delete m_completion;
-
-    delete m_saveTimer;
-
-    delete m_historyModel;
+        
     delete m_historyFilterModel;
     delete m_historyTreeModel;
-}
 
-
-QList<HistoryItem> HistoryManager::history() const
-{
-    return m_history;
+    delete m_saveTimer;
 }
 
 
@@ -119,6 +104,10 @@ bool HistoryManager::historyContains(const QString &url) const
 
 void HistoryManager::addHistoryEntry(const QString &url)
 {
+    QWebSettings *globalSettings = QWebSettings::globalSettings();
+    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
+        return;
+    
     QUrl cleanUrl(url);
 
     // don't store about: urls (home page related)
@@ -128,20 +117,19 @@ void HistoryManager::addHistoryEntry(const QString &url)
     cleanUrl.setPassword(QString());
     cleanUrl.setHost(cleanUrl.host().toLower());
     HistoryItem item(cleanUrl.toString(), QDateTime::currentDateTime());
-    addHistoryEntry(item);
 
-    // Add item to completion object
-    QString _url(url);
-    _url.remove(QRegExp("^http://|/$"));
-    UrlSearchItem urlSearchItem(UrlSearchItem::History, _url, QString(), item.dateTime, 1, QString(), QString());
-    m_completion->addItem(urlSearchItem);
+    m_history.prepend(item);
+    emit entryAdded(item);
+
+    if (m_history.count() == 1)
+        checkForExpired();
 }
 
 
 void HistoryManager::setHistory(const QList<HistoryItem> &history, bool loadedAndSorted)
 {
     m_history = history;
-
+ 
     // verify that it is sorted by date
     if (!loadedAndSorted)
         qSort(m_history.begin(), m_history.end());
@@ -161,24 +149,6 @@ void HistoryManager::setHistory(const QList<HistoryItem> &history, bool loadedAn
 }
 
 
-HistoryModel *HistoryManager::historyModel() const
-{
-    return m_historyModel;
-}
-
-
-HistoryFilterModel *HistoryManager::historyFilterModel() const
-{
-    return m_historyFilterModel;
-}
-
-
-HistoryTreeModel *HistoryManager::historyTreeModel() const
-{
-    return m_historyTreeModel;
-}
-
-
 void HistoryManager::checkForExpired()
 {
     if (m_historyLimit < 0 || m_history.isEmpty())
@@ -209,34 +179,32 @@ void HistoryManager::checkForExpired()
     }
 
     if (nextTimeout > 0)
-        m_expiredTimer.start(nextTimeout * 1000);
-}
-
-
-void HistoryManager::addHistoryEntry(const HistoryItem &item)
-{
-    QWebSettings *globalSettings = QWebSettings::globalSettings();
-    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
-        return;
-
-    m_history.prepend(item);
-    emit entryAdded(item);
-
-    if (m_history.count() == 1)
-        checkForExpired();
+        QTimer::singleShot( nextTimeout * 1000, this, SLOT(checkForExpired()) );
 }
 
 
 void HistoryManager::updateHistoryEntry(const KUrl &url, const QString &title)
 {
+    QString urlString = url.url();
+    urlString.remove(QL1S("www."));
+    if(urlString.startsWith(QL1S("http")) && urlString.endsWith(QL1C('/')))
+        urlString.remove(urlString.length()-1,1);
+    
     for (int i = 0; i < m_history.count(); ++i)
     {
-        if (url == m_history.at(i).url)
+        QString itemUrl = m_history.at(i).url;
+        itemUrl.remove(QL1S("www."));
+        if(itemUrl.startsWith(QL1S("http")) && itemUrl.endsWith(QL1C('/')))
+            itemUrl.remove(itemUrl.length()-1,1);
+        
+        if (urlString == itemUrl)
         {
             m_history[i].title = title;
+            m_history[i].url = url.url();
             m_saveTimer->changeOccurred();
             if (m_lastSavedUrl.isEmpty())
                 m_lastSavedUrl = m_history.at(i).url;
+            
             emit entryUpdated(i);
             break;
         }
@@ -244,14 +212,6 @@ void HistoryManager::updateHistoryEntry(const KUrl &url, const QString &title)
 }
 
 
-void HistoryManager::removeHistoryEntry(const HistoryItem &item)
-{
-    m_lastSavedUrl.clear();
-    m_history.removeOne(item);
-    emit entryRemoved(item);
-}
-
-
 void HistoryManager::removeHistoryEntry(const KUrl &url, const QString &title)
 {
     HistoryItem item;
@@ -261,30 +221,40 @@ void HistoryManager::removeHistoryEntry(const KUrl &url, const QString &title)
                 && (title.isEmpty() || title == m_history.at(i).title))
         {
             item = m_history.at(i);
-            removeHistoryEntry(item);
+            m_lastSavedUrl.clear();
+            m_history.removeOne(item);
+            emit entryRemoved(item);
             break;
         }
     }
-
-    // Remove item from completion object
-    UrlSearchItem urlSearchItem(UrlSearchItem::History, item.url, item.title, item.dateTime, 0, QString(), QString());
-    m_completion->removeItem(urlSearchItem);
 }
 
 
-int HistoryManager::historyLimit() const
+QList<HistoryItem> HistoryManager::find(const QString &text)
 {
-    return m_historyLimit;
-}
+    QList<HistoryItem> list;
 
+    QStringList urlKeys = m_historyFilterModel->keys();
+    Q_FOREACH(const QString &url, urlKeys)
+    {
+        int index = m_historyFilterModel->historyLocation(url);
+        HistoryItem item = m_history.at(index);
 
-void HistoryManager::setHistoryLimit(int limit)
-{
-    if (m_historyLimit == limit)
-        return;
-    m_historyLimit = limit;
-    checkForExpired();
-    m_saveTimer->changeOccurred();
+        QStringList words = text.split(' ');
+        bool matches = true;
+        foreach (const QString &word, words)
+        {
+            if (!url.contains(word, Qt::CaseInsensitive)
+                && !item.title.contains(word, Qt::CaseInsensitive)) {
+                matches = false;
+                break;
+            }
+        }
+        if (matches)
+            list << item;
+    }
+    
+    return list;
 }
 
 
@@ -304,13 +274,27 @@ void HistoryManager::loadSettings()
     int days;
     switch (historyExpire)
     {
-    case 0: days = 1; break;
-    case 1: days = 7; break;
-    case 2: days = 14; break;
-    case 3: days = 30; break;
-    case 4: days = 365; break;
-    case 5: days = -1; break;
-    default: days = -1;
+    case 0: 
+        days = 1; 
+        break;
+    case 1: 
+        days = 7; 
+        break;
+    case 2: 
+        days = 14; 
+        break;
+    case 3: 
+        days = 30; 
+        break;
+    case 4: 
+        days = 365; 
+        break;
+    case 5: 
+        days = -1; 
+        break;
+    default: 
+        days = -1;
+        break;
     }
     m_historyLimit = days;
 }
@@ -369,12 +353,6 @@ void HistoryManager::load()
 
         list.prepend(item);
         lastInsertedItem = item;
-
-        // Add item to completion object
-        //QString _url = item.url;
-        //_url.remove(QRegExp("^http://|/$"));
-        UrlSearchItem urlSearchItem(UrlSearchItem::History, item.url, item.title, item.dateTime, 1, QString(), QString());
-        m_completion->addItem(urlSearchItem);
     }
     if (needToSort)
         qSort(list.begin(), list.end());
@@ -456,72 +434,3 @@ void HistoryManager::save()
     }
     m_lastSavedUrl = m_history.value(0).url;
 }
-
-
-AwesomeUrlCompletion * HistoryManager::completionObject() const
-{
-    return m_completion;
-}
-
-
-void HistoryManager::addDownload(const QString &srcUrl, const QString &destUrl)
-{
-    QWebSettings *globalSettings = QWebSettings::globalSettings();
-    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
-        return;
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    if (!downloadFile.open(QFile::WriteOnly | QFile::Append))
-    {
-        kDebug() << "Unable to open download file (WRITE mode)..";
-        return;
-    }
-    QDataStream out(&downloadFile);
-    out << srcUrl;
-    out << destUrl;
-    out << QDateTime::currentDateTime();
-    downloadFile.close();
-}
-
-
-DownloadList HistoryManager::downloads()
-{
-    DownloadList list;
-
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    if (!downloadFile.open(QFile::ReadOnly))
-    {
-        kDebug() << "Unable to open download file (READ mode)..";
-        return list;
-    }
-
-    QDataStream in(&downloadFile);
-    while (!in.atEnd())
-    {
-        QString srcUrl;
-        in >> srcUrl;
-        QString destUrl;
-        in >> destUrl;
-        QDateTime dt;
-        in >> dt;
-        DownloadItem item(srcUrl, destUrl, dt);
-        list << item;
-    }
-    return list;
-}
-
-
-bool HistoryManager::clearDownloadsHistory()
-{
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    return downloadFile.remove();
-}
-
-
-QString HistoryManager::titleForHistoryUrl(QString url)
-{
-    return history().at(m_historyFilterModel->historyLocation(url)).title;
-}
-
