@@ -32,6 +32,8 @@
 
 // Rekonq Includes
 #include "rekonq_defines.h"
+
+// Local Includes
 #include "urlresolver.h"
 
 // KDE Includes
@@ -84,33 +86,8 @@ public:
 // ---------------------------------------------------------------------------------------------------------------
 
 
-class DownloadItem
-{
-public:
-    DownloadItem() {}
-    explicit DownloadItem(const QString &srcUrl,
-                          const QString &destUrl,
-                          const QDateTime &d
-                         )
-            : srcUrlString(srcUrl)
-            , destUrlString(destUrl)
-            , dateTime(d)
-    {}
-
-    QString srcUrlString;
-    QString destUrlString;
-    QDateTime dateTime;
-};
-
-
-typedef QList<DownloadItem> DownloadList;
-
-// ---------------------------------------------------------------------------------------------------------------
-
-
 // Forward Declarations
 class AutoSaver;
-class HistoryModel;
 class HistoryFilterModel;
 class HistoryTreeModel;
 
@@ -122,7 +99,6 @@ class HistoryTreeModel;
 class REKONQ_TESTS_EXPORT HistoryManager : public QWebHistoryInterface
 {
     Q_OBJECT
-    Q_PROPERTY(int historyLimit READ historyLimit WRITE setHistoryLimit)
 
 signals:
     void historyReset();
@@ -139,27 +115,14 @@ public:
     void updateHistoryEntry(const KUrl &url, const QString &title);
     void removeHistoryEntry(const KUrl &url, const QString &title = QString());
 
-    QString titleForHistoryUrl(QString url);
+    QList<HistoryItem> find(const QString &text);
 
-    int historyLimit() const;
-    void setHistoryLimit(int limit);
-
-    QList<HistoryItem> history() const;
+    QList<HistoryItem> history() const { return m_history; };
     void setHistory(const QList<HistoryItem> &history, bool loadedAndSorted = false);
 
     // History manager keeps around these models for use by the completer and other classes
-    HistoryModel *historyModel() const;
-    HistoryFilterModel *historyFilterModel() const;
-    HistoryTreeModel *historyTreeModel() const;
-
-    /**
-    * @returns the AwesomeUrlCompletion object.
-    */
-    AwesomeUrlCompletion *completionObject() const;
-
-    void addDownload(const QString &srcUrl, const QString &destUrl);
-    DownloadList downloads();
-    bool clearDownloadsHistory();
+    HistoryFilterModel *historyFilterModel() const { return m_historyFilterModel; };
+    HistoryTreeModel *historyTreeModel() const { return m_historyTreeModel; };
 
 public slots:
     void clear();
@@ -169,25 +132,16 @@ private slots:
     void save();
     void checkForExpired();
 
-protected:
-    void addHistoryEntry(const HistoryItem &item);
-    void removeHistoryEntry(const HistoryItem &item);
-
 private:
     void load();
 
     AutoSaver *m_saveTimer;
     int m_historyLimit;
-    QTimer m_expiredTimer;
     QList<HistoryItem> m_history;
     QString m_lastSavedUrl;
 
-    HistoryModel *m_historyModel;
     HistoryFilterModel *m_historyFilterModel;
     HistoryTreeModel *m_historyTreeModel;
-
-    // the completion object we sync with
-    AwesomeUrlCompletion *m_completion;
 };
 
 
