@@ -34,128 +34,56 @@
 
 // Local Includes
 #include "application.h"
+#include "paneltreeview.h"
 #include "historymodels.h"
-
-// Qt Includes
-#include <QtGui/QLabel>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QHeaderView>
-#include <QtGui/QTreeView>
-
+#include "urlfilterproxymodel.h"
 
 // KDE Includes
-#include <KLineEdit>
 #include <KLocalizedString>
 #include <KMenu>
 #include <KAction>
 #include <KMessageBox>
 
+// Qt Includes
+#include <QtGui/QHeaderView>
+
 
 HistoryPanel::HistoryPanel(const QString &title, QWidget *parent, Qt::WindowFlags flags)
-        : QDockWidget(title, parent, flags)
-        , m_treeView(new PanelTreeView(this))
-        , _loaded(false)
+        : UrlPanel(title, parent, flags)
 {
     setObjectName("historyPanel");
-    setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
-    
-    connect(this, SIGNAL(visibilityChanged(bool)), this, SLOT(showing(bool)));
-    
-    setShown(ReKonfig::showHistoryPanel());
+    setVisible(ReKonfig::showHistoryPanel());
 }
 
 
 HistoryPanel::~HistoryPanel()
 {
-    // Save side panel's state
     ReKonfig::setShowHistoryPanel(!isHidden());
 }
 
 
-
-void HistoryPanel::showing(bool b)
-{
-    if(b && !_loaded)
-        setup();
-}
-
-
-void HistoryPanel::setup()
-{
-    kDebug() << "Loading history panel setup...";
-
-    QWidget *ui = new QWidget(this);
-
-    m_treeView->setUniformRowHeights(true);
-    m_treeView->setSelectionBehavior(QAbstractItemView::SelectRows);
-    m_treeView->setTextElideMode(Qt::ElideMiddle);
-    m_treeView->setAlternatingRowColors(true);
-    m_treeView->header()->hide();
-
-    // add search bar
-    QHBoxLayout *hBoxLayout = new QHBoxLayout;
-    hBoxLayout->setContentsMargins(5, 0, 0, 0);
-    QLabel *searchLabel = new QLabel(i18n("Search:"));
-    hBoxLayout->addWidget(searchLabel);
-    KLineEdit *search = new KLineEdit;
-    search->setClearButtonShown(true);
-    hBoxLayout->addWidget(search);
-    QWidget *searchBar = new QWidget;
-    searchBar->setLayout(hBoxLayout);
-
-    // setup layout
-    QVBoxLayout *vBoxLayout = new QVBoxLayout;
-    vBoxLayout->setContentsMargins(0, 0, 0, 0);
-    vBoxLayout->addWidget(searchBar);
-    vBoxLayout->addWidget(m_treeView);
-
-    // add it to the UI
-    ui->setLayout(vBoxLayout);
-    setWidget(ui);
-
-    //-
-    HistoryManager *historyManager = Application::historyManager();
-    QAbstractItemModel *model = historyManager->historyTreeModel();
-
-    TreeProxyModel *treeProxyModel = new TreeProxyModel(this);
-    treeProxyModel->setSourceModel(model);
-    m_treeView->setModel(treeProxyModel);
-    m_treeView->setExpanded(treeProxyModel->index(0, 0), true);
-    m_treeView->header()->hideSection(1);
-    QFontMetrics fm(font());
-    int header = fm.width( QL1C('m') ) * 40;
-    m_treeView->header()->resizeSection(0, header);
-
-    connect(search, SIGNAL(textChanged(QString)), treeProxyModel, SLOT(setFilterFixedString(QString)));
-    connect(m_treeView, SIGNAL(contextMenuItemRequested(const QPoint &)), this, SLOT(contextMenuItem(const QPoint &)));
-    connect(m_treeView, SIGNAL(contextMenuGroupRequested(const QPoint &)), this, SLOT(contextMenuGroup(const QPoint &)));
-
-    _loaded = true;
-}
-
-
 void HistoryPanel::contextMenuItem(const QPoint &pos)
 {
     KMenu menu;
     KAction* action;
 
     action = new KAction(KIcon("tab-new"), i18n("Open"), this);
-    connect(action, SIGNAL(triggered()), m_treeView, SLOT(openInCurrentTab()));
+    connect(action, SIGNAL(triggered()), panelTreeView(), SLOT(openInCurrentTab()));
     menu.addAction(action);
 
     action = new KAction(KIcon("tab-new"), i18n("Open in New Tab"), this);
-    connect(action, SIGNAL(triggered()), m_treeView, SLOT(openInNewTab()));
+    connect(action, SIGNAL(triggered()), panelTreeView(), SLOT(openInNewTab()));
     menu.addAction(action);
 
     action = new KAction(KIcon("window-new"), i18n("Open in New Window"), this);
-    connect(action, SIGNAL(triggered()), m_treeView, SLOT(openInNewWindow()));
+    connect(action, SIGNAL(triggered()), panelTreeView(), SLOT(openInNewWindow()));
     menu.addAction(action);
 
     action = new KAction(KIcon("edit-copy"), i18n("Copy Link Address"), this);
-    connect(action, SIGNAL(triggered()), m_treeView, SLOT(copyToClipboard()));
+    connect(action, SIGNAL(triggered()), panelTreeView(), SLOT(copyToClipboard()));
     menu.addAction(action);
 
-    menu.exec(m_treeView->mapToGlobal(pos));
+    menu.exec(panelTreeView()->mapToGlobal(pos));
 }
 
 
@@ -168,13 +96,18 @@ void HistoryPanel::contextMenuGroup(const QPoint &pos)
     connect(action, SIGNAL(triggered()), this, SLOT(openAll()));
     menu.addAction(action);
 
-    menu.exec(m_treeView->mapToGlobal(pos));
+    menu.exec(panelTreeView()->mapToGlobal(pos));
+}
+
+
+void HistoryPanel::contextMenuEmpty(const QPoint& /*pos*/)
+{
 }
 
 
 void HistoryPanel::openAll()
 {
-    QModelIndex index = m_treeView->currentIndex();
+    QModelIndex index = panelTreeView()->currentIndex();
     if (!index.isValid())
         return;
 
@@ -198,3 +131,20 @@ void HistoryPanel::openAll()
         emit openUrl(allChild.at(i).url(), Rekonq::NewTab);
 }
 
+
+void HistoryPanel::setup()
+{
+    UrlPanel::setup();
+    kDebug() << "History panel...";
+
+    panelTreeView()->header()->hideSection(1);
+
+    const UrlFilterProxyModel *proxy = static_cast<const UrlFilterProxyModel*>(panelTreeView()->model());
+    panelTreeView()->expand(proxy->index(0, 0));
+}
+
+
+QAbstractItemModel* HistoryPanel::getModel()
+{
+    return Application::historyManager()->historyTreeModel();
+}
