@@ -23,20 +23,16 @@
 *
 * ============================================================ */
 
-// workaround for recent flashplugins
-#include <gtk/gtk.h>
 
 // version include
 #include "../version.h"
 
 // Local Includes
 #include "application.h"
-#include "sessionmanager.h"
 
 // KDE Includes
 #include <KAboutData>
 #include <KCmdLineArgs>
-#include <KDebug>
 
 
 static const char description[] =
@@ -45,9 +41,6 @@ static const char description[] =
 
 extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 {
-    // just a hackisch workaround for recent flashplugins
-    gtk_init(&argc, &argv);
-
     KAboutData about("rekonq",
                      0,
                      ki18n("rekonq"),
@@ -74,7 +67,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     ki18n("Developer, Ideas, Mockups, rekonq Icon"),
                     "megabigbug@yahoo.fr",
                     "");
-    
+
     about.addAuthor(ki18n("Johannes Zellner"),
                     ki18n("Patches, Suggestions, Testing, Bugfixing"),
                     "webmaster@nebulon.de",
@@ -106,7 +99,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "");
 
     about.addAuthor(ki18n("Rohan Garg"),
-                    ki18n("Handbook, Maintains a Kubuntu PPA with rekonq git packages"),
+                    ki18n("Handbook, Maintains a Kubuntu PPA with rekonq git packages. And now he also started coding.."),
                     "rohan16garg@gmail.com",
                     "");
 
@@ -114,18 +107,28 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     ki18n("Handbook, Maintains a Kubuntu PPA with rekonq git packages"),
                     "bulldog98@freenet.de",
                     "");
-                    
+
+    about.addAuthor(ki18n("Jon Ander Peñalba"),
+                    ki18n("Bookmarks code peer reviewer. A fantastic help"),
+                    "jonan88@gmail.com",
+                    "http://identi.ca/jonan");
+
+    about.addAuthor(ki18n("Benjamin Poulain"),
+                    ki18n("The \"QtWebKit guy\". Code quality improvements"),
+                    "ikipou@gmail.com",
+                    "http://www.openyourcode.org/");
+
     // --------------- about credits -----------------------------
     about.addCredit(ki18n("Dawit Alemayehu"),
                     ki18n("KDEWebKit (main) developer. And KIO. And KUriFilter. And more.."),
                     "adawit@kde.org",
                     "");
-                    
+
     about.addCredit(ki18n("Lindsay Mathieson"),
                     ki18n("Provided Patches & Hints, Discovered Bugs"),
                     "lindsay.mathieson@gmail.com",
                     "");
-                    
+
     about.addCredit(ki18n("Abdurrahman AVCI"),
                     ki18n("Provided Patches & Hints"),
                     "abdurrahmanavci@gmail.com",
@@ -186,7 +189,10 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
         kWarning() << "rekonq is already running!";
         return 0;
     }
-
+#if defined(Q_WS_X11)
+    // On X11, the raster engine gives better performance than native.
+    QApplication::setGraphicsSystem(QString::fromLatin1("raster"));
+#endif
     Application app;
 
     return app.exec();
