@@ -35,29 +35,26 @@
 #include "rekonq.h"
 
 // Local Includes
+#include "application.h"
+#include "iconmanager.h"
+#include "mainwindow.h"
+#include "sessionmanager.h"
+#include "stackedurlbar.h"
 #include "tabbar.h"
 #include "urlbar.h"
-#include "sessionmanager.h"
+#include "webpage.h"
+#include "webtab.h"
 
 // KDE Includes
-#include <KUrl>
 #include <KAction>
-#include <KShortcut>
-#include <KStandardShortcut>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardDirs>
-#include <KPassivePopup>
-#include <KLocalizedString>
 
 // Qt Includes
-#include <QTimer>
-#include <QString>
-#include <QAction>
-#include <QIcon>
-#include <QLabel>
-#include <QMovie>
-#include <QWidget>
-#include <QVBoxLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QMovie>
+#include <QtGui/QToolButton>
 
 
 MainView::MainView(MainWindow *parent)
@@ -119,7 +116,7 @@ void MainView::postLaunch()
         m_recentlyClosedTabs.removeAll(item);
         m_recentlyClosedTabs.prepend(item);
     }
-    
+
     // Session Manager
     connect(this, SIGNAL(tabsChanged()), Application::sessionManager(), SLOT(saveSession()));
     connect(this, SIGNAL(currentChanged(int)), Application::sessionManager(), SLOT(saveSession()));
@@ -293,10 +290,6 @@ void MainView::currentChanged(int index)
     else
         emit browserTabLoading(true);
 
-    // update zoom slider
-    if (!Application::instance()->mainWindowList().isEmpty())
-        Application::instance()->mainWindow()->setZoomSliderFactor(tab->view()->zoomFactor());
-
     // set focus to the current webview
     if (tab->url().scheme() == QL1S("about"))
         _widgetBar->currentWidget()->setFocus();
@@ -326,10 +319,9 @@ WebTab *MainView::newWebTab(bool focused)
     // connecting webview with mainview
     connect(tab->view(), SIGNAL(loadStarted()), this, SLOT(webViewLoadStarted()));
     connect(tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(webViewLoadFinished(bool)));
-    connect(tab->view(), SIGNAL(iconChanged()), this, SLOT(webViewIconChanged()));
     connect(tab->view(), SIGNAL(titleChanged(const QString &)), this, SLOT(webViewTitleChanged(const QString &)));
     connect(tab->view(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(webViewUrlChanged(const QUrl &)));
-    connect(tab->view(), SIGNAL(zoomChanged(qreal)), m_parentWindow, SLOT(setZoomSliderFactor(qreal)));
+    connect(tab->view(), SIGNAL(iconChanged()), this, SLOT(webViewIconChanged()));
 
     // connecting webPage signals with mainview
     connect(tab->view()->page(), SIGNAL(windowCloseRequested()), this, SLOT(windowCloseRequested()));
@@ -417,7 +409,7 @@ void MainView::closeOtherTabs(int index)
         index = currentIndex();
     if (index < 0 || index >= count())
         return;
-    
+
     for (int i = count() - 1; i > index; --i)
     {
         closeTab(i);
@@ -439,14 +431,9 @@ void MainView::cloneTab(int index)
     if (index < 0 || index >= count())
         return;
 
-    WebTab *tab = newWebTab();
     KUrl url = webTab(index)->url();
 
-    // workaround against bug in webkit:
-    // only set url if it is not empty
-    // otherwise the current working directory will be used
-    if (!url.isEmpty())
-        tab->view()->setUrl(url);
+    Application::instance()->loadUrl(url, Rekonq::NewTab);
 
     updateTabBar();
 }
@@ -459,10 +446,10 @@ void MainView::closeTab(int index, bool del)
     if (count() == 1)
     {
         WebView *w = currentWebTab()->view();
-        
+
         if( currentWebTab()->url().protocol() == QL1S("about") )
             return;
-        
+
         switch (ReKonfig::newTabsBehaviour())
         {
         case 0: // new tab page
@@ -499,7 +486,9 @@ void MainView::closeTab(int index, bool del)
             return;
     }
 
-    if (!tab->url().isEmpty())
+    if (!tab->url().isEmpty()
+        && !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)
+       )
     {
         QString title = tab->view()->title();
         QString url = tab->url().prettyUrl();
@@ -584,7 +573,7 @@ void MainView::webViewIconChanged()
     int index = indexOf(view->parentWidget());
     if (-1 != index)
     {
-        KIcon icon = Application::icon(view->url());
+        KIcon icon = Application::iconManager()->iconForUrl(view->url());
         QLabel *label = animatedLoading(index, false);
         QMovie *movie = label->movie();
         delete movie;
@@ -599,7 +588,7 @@ void MainView::webViewTitleChanged(const QString &title)
     QString viewTitle = title.isEmpty()? i18n("(Untitled)") : title;
     QString tabTitle = viewTitle;
     tabTitle.replace('&', "&&");
-    
+
     WebView *view = qobject_cast<WebView *>(sender());
     int index = indexOf(view->parentWidget());
     if (-1 != index)
@@ -643,6 +632,7 @@ void MainView::previousTab()
     setCurrentIndex(next);
 }
 
+
 void MainView::openClosedTabs()
 {
     foreach (const HistoryItem &item, recentlyClosedTabs())
@@ -652,13 +642,14 @@ void MainView::openClosedTabs()
     m_recentlyClosedTabs.clear();
 }
 
+
 void MainView::openClosedTab()
 {
     KAction *action = qobject_cast<KAction *>(sender());
     if (action)
     {
         Application::instance()->loadUrl(action->data().toUrl(), Rekonq::NewTab);
-        
+
         QString title = action->text();
         title = title.remove('&');
         HistoryItem item(action->data().toString(), QDateTime(), title );
@@ -666,6 +657,7 @@ void MainView::openClosedTab()
     }
 }
 
+
 void MainView::switchToTab()
 {
     // uses the sender to determine the tab index
@@ -677,6 +669,7 @@ void MainView::switchToTab()
     setCurrentIndex( index );
 }
 
+
 QLabel *MainView::animatedLoading(int index, bool addMovie)
 {
     if (index == -1)
@@ -707,7 +700,7 @@ void MainView::resizeEvent(QResizeEvent *event)
 }
 
 
-void MainView::detachTab(int index)
+void MainView::detachTab(int index, MainWindow *toWindow)
 {
     if (index < 0)
         index = currentIndex();
@@ -728,8 +721,12 @@ void MainView::detachTab(int index)
         UrlBar *bar = _widgetBar->urlBar(index);
         closeTab(index, false);
 
-        MainWindow *w = Application::instance()->newMainWindow(false);
-        w->mainView()->addTab(tab, Application::icon(u), label);
+        MainWindow *w;
+        if( toWindow == NULL )
+            w = Application::instance()->newMainWindow(false);
+        else
+            w = toWindow;
+        w->mainView()->addTab(tab, Application::iconManager()->iconForUrl(u), label);
         w->mainView()->widgetBar()->insertWidget(0, bar);
         w->mainView()->updateTabBar();
     }
