@@ -35,28 +35,23 @@
 #include "rekonq_defines.h"
 
 // Local Includes
-#include "webview.h"
-#include "webpage.h"
-#include "webtab.h"
-#include "application.h"
 #include "historymanager.h"
-#include "mainwindow.h"
-#include "stackedurlbar.h"
 
 // KDE Includes
 #include <KTabWidget>
 
-// Qt Includes
-#include <QtGui/QToolButton>
-
 // Forward Declarations
-class QUrl;
-class QWebFrame;
-class QLabel;
-class QMouseEvent;
-
+class HistoryItem;
+class MainWindow;
+class StackedUrlBar;
 class TabBar;
 class UrlBar;
+class WebTab;
+
+class QLabel;
+class QToolButton;
+class QUrl;
+class QWebFrame;
 
 
 /**
@@ -129,12 +124,17 @@ public slots:
     void closeTab(int index = -1, bool del = true);
     void closeOtherTabs(int index = -1);
     void reloadTab(int index = -1);
-    void detachTab(int index = -1);
-    
+    /**
+     * Detaches tab at @c index to a new window.
+     * If @c toWindow is not null, the tab is instead
+     * added to existing MainWindow @c toWindow.
+     */
+    void detachTab(int index = -1, MainWindow *toWindow = NULL);
+
     void reloadAllTabs();
     void nextTab();
     void previousTab();
-    
+
     void openClosedTabs();
     void openClosedTab();
     void switchToTab();
