@@ -35,84 +35,75 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "settingsdialog.h"
-#include "historymanager.h"
-#include "bookmarksmanager.h"
-#include "webtab.h"
-#include "mainview.h"
+#include "analyzerpanel.h"
+#include "application.h"
+#include "bookmarkprovider.h"
+#include "bookmarkspanel.h"
+#include "bookmarkstoolbar.h"
 #include "findbar.h"
 #include "historypanel.h"
-#include "bookmarkspanel.h"
-#include "webinspectorpanel.h"
-#include "urlbar.h"
+#include "iconmanager.h"
+#include "mainview.h"
+#include "settingsdialog.h"
+#include "stackedurlbar.h"
 #include "tabbar.h"
-#include "adblockmanager.h"
-#include "analyzerpanel.h"
+#include "urlbar.h"
+#include "webinspectorpanel.h"
+#include "webpage.h"
+#include "webtab.h"
+#include "zoombar.h"
 
 // Ui Includes
 #include "ui_cleardata.h"
 
 // KDE Includes
-#include <KShortcut>
-#include <KStandardAction>
+#include <KIO/Job>
+
 #include <KAction>
-#include <KToggleFullScreenAction>
-#include <KActionCollection>
-#include <KMessageBox>
+#include <KEditToolBar>
 #include <KFileDialog>
-#include <KGlobalSettings>
-#include <KPushButton>
-#include <KTemporaryFile>
-#include <KPassivePopup>
-#include <KMenuBar>
 #include <KJobUiDelegate>
-#include <kdeprintdialog.h>
-#include <KToggleAction>
-#include <KStandardDirs>
-#include <KActionCategory>
+#include <KMenu>
+#include <KMenuBar>
+#include <KMessageBox>
+#include <KPassivePopup>
 #include <KProcess>
+#include <KPushButton>
+#include <KStandardDirs>
+#include <KToggleFullScreenAction>
 
 // Qt Includes
 #include <QtCore/QTimer>
-#include <QtCore/QRect>
-#include <QtCore/QSize>
-#include <QtCore/QList>
-#include <QtCore/QWeakPointer>
-
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QFont>
-#include <QtGui/QDesktopWidget>
-#include <QtGui/QPrinter>
-#include <QtGui/QPrintDialog>
-#include <QtGui/QPrintPreviewDialog>
-#include <QtGui/QFontMetrics>
 
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusReply>
 
+#include <QtGui/QDesktopWidget>
+#include <QtGui/QLabel>
+#include <QtGui/QPrintDialog>
+#include <QtGui/QPrinter>
+#include <QtGui/QPrintPreviewDialog>
+#include <QtGui/QVBoxLayout>
+
 #include <QtWebKit/QWebHistory>
 
 
 MainWindow::MainWindow()
-        : KMainWindow()
+        : KXmlGuiWindow()
         , m_view(new MainView(this))
         , m_findBar(new FindBar(this))
+        , m_zoomBar(new ZoomBar(this))
         , m_historyPanel(0)
         , m_bookmarksPanel(0)
         , m_webInspectorPanel(0)
         , m_analyzerPanel(0)
         , m_historyBackMenu(0)
         , m_encodingMenu(new KMenu(this))
-        , m_mainBar(new KToolBar(QString("MainToolBar"), this, Qt::TopToolBarArea, true, true, true))
-        , m_bmBar(new KToolBar(QString("BookmarkToolBar"), this, Qt::TopToolBarArea, true, false, true))
+        , m_bookmarksBar(0)
         , m_popup(new KPassivePopup(this))
         , m_hidePopup(new QTimer(this))
-        , m_ac(new KActionCollection(this))
+        , m_toolsMenu(0)
 {
-    kDebug() << "MainWindow ctor...";
-    // enable window size "auto-save"
-    setAutoSaveSettings();
-
     // creating a centralWidget containing panel, m_view and the hidden findbar
     QWidget *centralWidget = new QWidget;
     centralWidget->setContentsMargins(0, 0, 0, 0);
@@ -122,6 +113,7 @@ MainWindow::MainWindow()
     layout->setContentsMargins(0, 0, 0, 0);
     layout->addWidget(m_view);
     layout->addWidget(m_findBar);
+    layout->addWidget(m_zoomBar);
     centralWidget->setLayout(layout);
 
     // central widget
@@ -142,17 +134,21 @@ MainWindow::MainWindow()
     // setting up rekonq toolbar(s)
     setupToolbars();
 
+    // a call to KXmlGuiWindow::setupGUI() populates the GUI
+    // with actions, using KXMLGUI.
+    // It also applies the saved mainwindow settings, if any, and ask the
+    // mainwindow to automatically save settings if changed: window size,
+    // toolbar position, icon size, etc.
+    setupGUI();
+
+    // no menu bar in rekonq: we have other plans..
+    menuBar()->setVisible(false);
+
     // no more status bar..
     setStatusBar(0);
 
-    // setting popup notification
-    m_popup->setAutoDelete(false);
-    connect(Application::instance(), SIGNAL(focusChanged(QWidget*, QWidget*)), m_popup, SLOT(hide()));
-    m_popup->setFrameShape(QFrame::NoFrame);
-    m_popup->setLineWidth(0);
-    connect(m_hidePopup, SIGNAL(timeout()), m_popup, SLOT(hide()));
-
-    QTimer::singleShot(0, this, SLOT(postLaunch()));
+    // give me some time to do all the other stuffs...
+    QTimer::singleShot(100, this, SLOT(postLaunch()));
 
     kDebug() << "MainWindow ctor...DONE";
 }
@@ -160,11 +156,13 @@ MainWindow::MainWindow()
 
 MainWindow::~MainWindow()
 {
-    Application::bookmarkProvider()->removeToolBar(m_bmBar);
+    Application::bookmarkProvider()->removeBookmarkBar(m_bookmarksBar);
+    Application::bookmarkProvider()->removeBookmarkPanel(m_bookmarksPanel);
     Application::instance()->removeMainWindow(this);
-    
+
     delete m_view;
     delete m_findBar;
+    delete m_zoomBar;
 
     delete m_historyPanel;
     delete m_bookmarksPanel;
@@ -174,63 +172,123 @@ MainWindow::~MainWindow()
     delete m_historyBackMenu;
     delete m_encodingMenu;
 
-    delete m_mainBar;
-    delete m_bmBar;
-
-    delete m_zoomSlider;
+    delete m_bookmarksBar;
 
     delete m_popup;
     delete m_hidePopup;
-
-    delete m_ac;
 }
 
 
 void MainWindow::setupToolbars()
 {
     kDebug() << "setup toolbars...";
-    
-    // ============ Main ToolBar  ================================
-    m_mainBar->addAction(actionByName(KStandardAction::name(KStandardAction::Back)));
-    m_mainBar->addAction(actionByName(KStandardAction::name(KStandardAction::Forward)));
-    m_mainBar->addSeparator();
-    m_mainBar->addAction(actionByName( QL1S("stop_reload") ));
-    m_mainBar->addAction(actionByName(KStandardAction::name(KStandardAction::Home)));
+
+    KAction *a;
 
     // location bar
-    KAction *urlBarAction = new KAction(this);
-    urlBarAction->setDefaultWidget(m_view->widgetBar());
-    m_mainBar->addAction(urlBarAction);
+    a = new KAction(i18n("Location Bar"), this);
+    a->setDefaultWidget(m_view->widgetBar());
+    actionCollection()->addAction( QL1S("url_bar"), a);
 
-    m_mainBar->addAction(actionByName( QL1S("bookmarksActionMenu") ));
-    m_mainBar->addAction(actionByName( QL1S("rekonq_tools") ));
+    KToolBar *mainBar = toolBar("mainToolBar");
 
-    m_mainBar->show();  // this just to fix reopening rekonq after fullscreen close
+    mainBar->show();  // this just to fix reopening rekonq after fullscreen close
+}
 
-    // =========== Bookmarks ToolBar ================================
-    m_bmBar->setAcceptDrops(true);
-    Application::bookmarkProvider()->setupBookmarkBar(m_bmBar);
 
-    if (ReKonfig::firstExecution())
+void MainWindow::initBookmarkBar()
+{
+    KToolBar *XMLGUIBkBar = toolBar("bookmarkToolBar");
+    if (!XMLGUIBkBar)
+        return;
+
+    if (m_bookmarksBar)
     {
-        m_mainBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
-
-        m_bmBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-        m_bmBar->setIconDimensions(16);
-        m_bmBar->hide();
-
-        KToolBar::setToolBarsEditable(false);
-        KToolBar::setToolBarsLocked(true);
-
-        ReKonfig::setFirstExecution(false);
+        Application::bookmarkProvider()->removeBookmarkBar(m_bookmarksBar);
+        delete m_bookmarksBar;
     }
+    m_bookmarksBar = new BookmarkToolBar(XMLGUIBkBar, this);
+    Application::bookmarkProvider()->registerBookmarkBar(m_bookmarksBar);
+
+    // To update the bookmark toolbar action
+    initToolsMenu();
+}
+
+
+void MainWindow::configureToolbars()
+{
+    if (autoSaveSettings())
+        saveAutoSaveSettings();
+
+    KEditToolBar dlg(factory(), this);
+    // The bookmark bar needs to be refill after the UI changes are finished
+    connect(&dlg, SIGNAL(newToolBarConfig()), this, SLOT(initBookmarkBar()));
+    dlg.exec();
+}
+
+
+void MainWindow::initToolsMenu()
+{
+    if (!m_toolsMenu)
+        return;
+
+    m_toolsMenu->menu()->clear();
+
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Open)));
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::SaveAs)));
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Print)));
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Find)));
+
+    QAction *action = actionByName(KStandardAction::name(KStandardAction::Zoom));
+    action->setCheckable(true);
+    connect (m_zoomBar, SIGNAL(visibilityChanged(bool)), action, SLOT(setChecked(bool)));
+    m_toolsMenu->addAction(action);
+
+    m_toolsMenu->addAction(actionByName(QL1S("encodings")));
+
+    m_toolsMenu->addSeparator();
+
+    m_toolsMenu->addAction(actionByName(QL1S("private_browsing")));
+    m_toolsMenu->addAction(actionByName(QL1S("clear_private_data")));
+
+    m_toolsMenu->addSeparator();
+
+    KActionMenu *webMenu = new KActionMenu(KIcon("applications-development-web"), i18n("Development"), this);
+    webMenu->addAction(actionByName(QL1S("web_inspector")));
+    webMenu->addAction(actionByName(QL1S("page_source")));
+    webMenu->addAction(actionByName(QL1S("net_analyzer")));
+    m_toolsMenu->addAction(webMenu);
+
+    m_toolsMenu->addSeparator();
+
+    action = m_bookmarksBar->toolBar()->toggleViewAction();
+    action->setText(i18n("Bookmarks Toolbar"));
+    action->setIcon(KIcon("bookmarks-bar"));
+    m_toolsMenu->addAction(action);
+
+    m_toolsMenu->addAction(actionByName(QL1S("show_history_panel")));
+    m_toolsMenu->addAction(actionByName(QL1S("show_bookmarks_panel")));
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::FullScreen)));
+
+    m_toolsMenu->addSeparator();
+
+    this->setHelpMenuEnabled(true);
+    helpMenu()->setIcon(KIcon("help-browser"));
+    m_toolsMenu->addAction(helpMenu()->menuAction());
+    m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Preferences)));
 }
 
 
 void MainWindow::postLaunch()
 {
-    // KActionCollection read settings
-    m_ac->readSettings();
+    setupBookmarksAndToolsShortcuts();
+
+    // setting popup notification
+    m_popup->setAutoDelete(false);
+    connect(Application::instance(), SIGNAL(focusChanged(QWidget*, QWidget*)), m_popup, SLOT(hide()));
+    m_popup->setFrameShape(QFrame::NoFrame);
+    m_popup->setLineWidth(0);
+    connect(m_hidePopup, SIGNAL(timeout()), m_popup, SLOT(hide()));
 
     // notification system
     connect(m_view, SIGNAL(showStatusBarMessage(const QString&, Rekonq::Notify)), this, SLOT(notifyMessage(const QString&, Rekonq::Notify)));
@@ -254,11 +312,19 @@ void MainWindow::postLaunch()
     // Find Bar signal
     connect(m_findBar, SIGNAL(searchString(const QString &)), this, SLOT(find(const QString &)));
 
+    // Zoom Bar signal
+    connect(m_view, SIGNAL(currentChanged(int)), m_zoomBar, SLOT(updateSlider(int)));
+    // Ctrl + wheel handling
+    connect(this->currentTab()->view(), SIGNAL(zoomChanged(int)), m_zoomBar, SLOT(setValue(int)));
+
     // setting up toolbars to NOT have context menu enabled
     setContextMenuPolicy(Qt::DefaultContextMenu);
 
     // accept d'n'd
     setAcceptDrops(true);
+
+    // Bookmark ToolBar (needs to be setup after the call to setupGUI())
+    initBookmarkBar();
 }
 
 
@@ -273,7 +339,7 @@ QSize MainWindow::sizeHint() const
 void MainWindow::setupActions()
 {
     kDebug() << "setup actions...";
-    
+
     // this let shortcuts work..
     actionCollection()->addAssociatedWidget(this);
 
@@ -333,12 +399,8 @@ void MainWindow::setupActions()
     actionCollection()->addAction(QL1S("open_location"), a);
     connect(a, SIGNAL(triggered(bool)) , this, SLOT(openLocation()));
 
-
-    // ============================= Zoom Actions ===================================
-    actionCollection()->addAction(KStandardAction::ZoomIn, this, SLOT(zoomIn()));
-    a = actionCollection()->addAction(KStandardAction::ActualSize, this, SLOT(zoomNormal()));
-    a->setShortcut(KShortcut(Qt::CTRL | Qt::Key_0));
-    actionCollection()->addAction(KStandardAction::ZoomOut, this, SLOT(zoomOut()));
+    // set zoom bar actions
+    m_zoomBar->setupActions(this);
 
     // =============================== Tools Actions =================================
     a = new KAction(i18n("Page S&ource"), this);
@@ -386,7 +448,7 @@ void MainWindow::setupActions()
     a->setShortcuts(QApplication::isRightToLeft() ? KStandardShortcut::tabNext() : KStandardShortcut::tabPrev());
     actionCollection()->addAction(QL1S("show_prev_tab"), a);
     connect(a, SIGNAL(triggered(bool)), m_view, SLOT(previousTab()));
-    
+
     a = new KAction(KIcon("tab-new"), i18n("Open Closed Tabs"), this);
     a->setShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_T));
     actionCollection()->addAction(QL1S("open_closed_tabs"), a);
@@ -406,7 +468,6 @@ void MainWindow::setupActions()
         connect(a, SIGNAL(triggered(bool)), m_view, SLOT(switchToTab()));
     }
 
-
     // ============================== Indexed Tab Actions ====================================
     a = new KAction(KIcon("tab-close"), i18n("&Close Tab"), this);
     a->setShortcuts( KStandardShortcut::close() );
@@ -429,27 +490,14 @@ void MainWindow::setupActions()
     actionCollection()->addAction(QL1S("detach_tab"), a);
     connect(a, SIGNAL(triggered(bool)), m_view->tabBar(), SLOT(detachTab()));
 
-
-    // ----------------------- Bookmarks ToolBar Action --------------------------------------
-    QAction *qa;
-
-    qa = m_mainBar->toggleViewAction();
-    qa->setText(i18n("Main Toolbar"));
-    qa->setIcon(KIcon("bookmark-toolbar"));
-    actionCollection()->addAction(QL1S("main_bar"), qa);
-
-    qa = m_bmBar->toggleViewAction();
-    qa->setText(i18n("Bookmarks Toolbar"));
-    qa->setIcon(KIcon("bookmark-toolbar"));
-    actionCollection()->addAction(QL1S("bm_bar"), qa);
-
     // Bookmark Menu
     KActionMenu *bmMenu = Application::bookmarkProvider()->bookmarkActionMenu(this);
     bmMenu->setIcon(KIcon("bookmarks"));
     bmMenu->setDelayed(false);
+    bmMenu->setShortcutConfigurable(true);
+    bmMenu->setShortcut( KShortcut(Qt::ALT + Qt::Key_B) );
     actionCollection()->addAction(QL1S("bookmarksActionMenu"), bmMenu);
 
-
     // ---------------- Encodings -----------------------------------
     a = new KAction(KIcon("character-set"), i18n("Set Encoding"), this);
     actionCollection()->addAction(QL1S("encodings"), a);
@@ -462,78 +510,15 @@ void MainWindow::setupActions()
 void MainWindow::setupTools()
 {
     kDebug() << "setup tools...";
-    KActionMenu *toolsMenu = new KActionMenu(KIcon("configure"), i18n("&Tools"), this);
-    toolsMenu->setDelayed(false);
-
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Open)));
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::SaveAs)));
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Print)));
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Find)));
-
-    // setup zoom widget
-    QWidget *zoomWidget = new QWidget(this);
-
-    QToolButton *zoomOut = new QToolButton(zoomWidget);
-    zoomOut->setDefaultAction(actionByName(KStandardAction::name(KStandardAction::ZoomOut)));
-    zoomOut->setAutoRaise(true);
-
-    m_zoomSlider = new QSlider(Qt::Horizontal, zoomWidget);
-    m_zoomSlider->setTracking(true);
-    m_zoomSlider->setRange(1, 19);      // divide by 10 to obtain a qreal for zoomFactor()
-    m_zoomSlider->setValue(10);
-    m_zoomSlider->setPageStep(3);
-    connect(m_zoomSlider, SIGNAL(valueChanged(int)), this, SLOT(setZoomFactor(int)));
-
-    QToolButton *zoomIn = new QToolButton(zoomWidget);
-    zoomIn->setDefaultAction(actionByName(KStandardAction::name(KStandardAction::ZoomIn)));
-    zoomIn->setAutoRaise(true);
-
-    QToolButton *zoomNormal = new QToolButton(zoomWidget);
-    zoomNormal->setDefaultAction(actionByName(KStandardAction::name(KStandardAction::ActualSize)));
-    zoomNormal->setAutoRaise(true);
-
-    QHBoxLayout* zoomWidgetLayout = new QHBoxLayout(zoomWidget);
-    zoomWidgetLayout->setSpacing(0);
-    zoomWidgetLayout->setMargin(0);
-    zoomWidgetLayout->addWidget(zoomOut);
-    zoomWidgetLayout->addWidget(m_zoomSlider);
-    zoomWidgetLayout->addWidget(zoomIn);
-    zoomWidgetLayout->addWidget(zoomNormal);
-
-    QWidgetAction *zoomAction = new QWidgetAction(this);
-    zoomAction->setDefaultWidget(zoomWidget);
-    toolsMenu->addAction(zoomAction);
-
-    toolsMenu->addAction(actionByName(QL1S("encodings")));
-
-    toolsMenu->addSeparator();
-
-    toolsMenu->addAction(actionByName(QL1S("private_browsing")));
-    toolsMenu->addAction(actionByName(QL1S("clear_private_data")));
-
-    toolsMenu->addSeparator();
-
-    KActionMenu *webMenu = new KActionMenu(KIcon("applications-development-web"), i18n("Development"), this);
-    webMenu->addAction(actionByName(QL1S("web_inspector")));
-    webMenu->addAction(actionByName(QL1S("page_source")));
-    webMenu->addAction(actionByName(QL1S("net_analyzer")));
-    toolsMenu->addAction(webMenu);
-
-    toolsMenu->addSeparator();
-
-    toolsMenu->addAction(actionByName(QL1S("bm_bar")));
-    toolsMenu->addAction(actionByName(QL1S("show_history_panel")));
-    toolsMenu->addAction(actionByName(QL1S("show_bookmarks_panel")));
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::FullScreen)));
-
-    toolsMenu->addSeparator();
-
-    helpMenu()->setIcon(KIcon("help-browser"));
-    toolsMenu->addAction(helpMenu()->menuAction());
-    toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Preferences)));
+    m_toolsMenu = new KActionMenu(KIcon("configure"), i18n("&Tools"), this);
+    m_toolsMenu->setDelayed(false);
+    m_toolsMenu->setShortcutConfigurable(true);
+    m_toolsMenu->setShortcut( KShortcut(Qt::ALT + Qt::Key_T) );
 
     // adding rekonq_tools to rekonq actionCollection
-    actionCollection()->addAction(QL1S("rekonq_tools"), toolsMenu);
+    actionCollection()->addAction(QL1S("rekonq_tools"), m_toolsMenu);
+
+    // Actions are added after the call to setupGUI() to ensure the help menu works
 }
 
 
@@ -566,6 +551,8 @@ void MainWindow::setupPanels()
 
     addDockWidget(Qt::LeftDockWidgetArea, m_bookmarksPanel);
 
+    Application::bookmarkProvider()->registerBookmarkPanel(m_bookmarksPanel);
+
     // setup bookmarks panel action
     a = (KAction *) m_bookmarksPanel->toggleViewAction();
     a->setShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_B));
@@ -584,7 +571,7 @@ void MainWindow::setupPanels()
 
     addDockWidget(Qt::BottomDockWidgetArea, m_webInspectorPanel);
     m_webInspectorPanel->hide();
-    
+
     // STEP 4
     // Setup Network analyzer panel
     m_analyzerPanel = new NetworkAnalyzerPanel( i18n("Network Analyzer"), this);
@@ -602,6 +589,10 @@ void MainWindow::setupPanels()
 
 void MainWindow::openLocation()
 {
+    if(isFullScreen())
+    {
+        setWidgetsVisible(true);
+    }
     m_view->urlBar()->selectAll();
     m_view->urlBar()->setFocus();
 }
@@ -621,7 +612,7 @@ void MainWindow::fileSaveAs()
         srcUrl = w->url();
     }
     kDebug() << "URL to save: " << srcUrl;
-    
+
     QString name = srcUrl.fileName();
     if (name.isNull())
     {
@@ -658,7 +649,7 @@ void MainWindow::updateActions()
 {
     kDebug() << "updating actions..";
     bool rekonqPage = currentTab()->page()->isOnRekonqPage();
-    
+
     QAction *historyBackAction = actionByName(KStandardAction::name(KStandardAction::Back));
     if( rekonqPage && currentTab()->view()->history()->count() > 0 )
         historyBackAction->setEnabled(true);
@@ -779,6 +770,7 @@ void MainWindow::find(const QString & search)
         return;
     m_lastSearch = search;
 
+    updateHighlight();
     findNext();
 }
 
@@ -790,6 +782,7 @@ void MainWindow::matchCaseUpdate()
 
     currentTab()->view()->findText(m_lastSearch, QWebPage::FindBackward);
     findNext();
+    updateHighlight();
 }
 
 
@@ -798,8 +791,6 @@ void MainWindow::findNext()
     if (!currentTab())
         return;
 
-    highlightAll();
-
     if (m_findBar->isHidden())
     {
         QPoint previous_position = currentTab()->view()->page()->currentFrame()->scrollPosition();
@@ -808,8 +799,6 @@ void MainWindow::findNext()
         return;
     }
 
-    highlightAll();
-
     QWebPage::FindFlags options = QWebPage::FindWrapsAroundDocument;
     if (m_findBar->matchCase())
         options |= QWebPage::FindCaseSensitively;
@@ -839,7 +828,8 @@ void MainWindow::findPrevious()
     m_findBar->notifyMatch(found);
 }
 
-void MainWindow::highlightAll()
+
+void MainWindow::updateHighlight()
 {
     if (!currentTab())
         return;
@@ -858,38 +848,6 @@ void MainWindow::highlightAll()
 }
 
 
-void MainWindow::zoomIn()
-{
-    m_zoomSlider->setValue(m_zoomSlider->value() + 1);
-}
-
-
-void MainWindow::zoomNormal()
-{
-    m_zoomSlider->setValue(10);
-}
-
-
-void MainWindow::zoomOut()
-{
-    m_zoomSlider->setValue(m_zoomSlider->value() - 1);
-}
-
-
-void MainWindow::setZoomFactor(int factor)
-{
-    if (!currentTab())
-        return;
-    currentTab()->view()->setZoomFactor(QVariant(factor).toReal() / 10);
-}
-
-
-void MainWindow::setZoomSliderFactor(qreal factor)
-{
-    m_zoomSlider->setValue(factor * 10);
-}
-
-
 void MainWindow::viewFullScreen(bool makeFullScreen)
 {
     setWidgetsVisible(!makeFullScreen);
@@ -904,33 +862,36 @@ void MainWindow::setWidgetsVisible(bool makeVisible)
     static bool historyPanelFlag;
     static bool bookmarksPanelFlag;
 
+    KToolBar *mainBar = toolBar("mainToolBar");
+    KToolBar *bookBar = toolBar("bookmarksToolBar");
+
     if (!makeVisible)
     {
         // save current state, if in windowed mode
         if (!isFullScreen())
         {
-            bookmarksToolBarFlag = m_bmBar->isHidden();
+            bookmarksToolBarFlag = bookBar->isHidden();
             historyPanelFlag = m_historyPanel->isHidden();
             bookmarksPanelFlag = m_bookmarksPanel->isHidden();
         }
 
-        m_bmBar->hide();
+        bookBar->hide();
         m_view->setTabBarHidden(true);
         m_historyPanel->hide();
         m_bookmarksPanel->hide();
 
         // hide main toolbar
-        m_mainBar->hide();
+        mainBar->hide();
     }
     else
     {
         // show main toolbar
-        m_mainBar->show();
+        mainBar->show();
         m_view->setTabBarHidden(false);
 
         // restore state of windowed mode
         if (!bookmarksToolBarFlag)
-            m_bmBar->show();
+            bookBar->show();
         if (!historyPanelFlag)
             m_historyPanel->show();
         if (!bookmarksPanelFlag)
@@ -939,6 +900,15 @@ void MainWindow::setWidgetsVisible(bool makeVisible)
 }
 
 
+QString MainWindow::selectedText() const
+{
+    if (!currentTab())
+        return QString();
+
+    return currentTab()->view()->selectedText();
+}
+
+
 void MainWindow::viewPageSource()
 {
     if (!currentTab())
@@ -951,10 +921,10 @@ void MainWindow::viewPageSource()
 
 void MainWindow::homePage(Qt::MouseButtons mouseButtons, Qt::KeyboardModifiers keyboardModifiers)
 {
-    KUrl homeUrl = ReKonfig::useNewTabPage() 
+    KUrl homeUrl = ReKonfig::useNewTabPage()
         ? KUrl( QL1S("about:home") )
         : KUrl( ReKonfig::homePage() );
-        
+
     if (mouseButtons == Qt::MidButton || keyboardModifiers == Qt::ControlModifier)
         Application::instance()->loadUrl( homeUrl, Rekonq::NewTab);
     else
@@ -996,7 +966,7 @@ void MainWindow::openPrevious(Qt::MouseButtons mouseButtons, Qt::KeyboardModifie
 {
     QWebHistory *history = currentTab()->view()->history();
     QWebHistoryItem *item = 0;
-    
+
     if (currentTab()->page()->isOnRekonqPage())
     {
         item = new QWebHistoryItem(history->currentItem());
@@ -1012,7 +982,7 @@ void MainWindow::openPrevious(Qt::MouseButtons mouseButtons, Qt::KeyboardModifie
 
     if(!item)
         return;
-    
+
     if (mouseButtons == Qt::MidButton || keyboardModifiers == Qt::ControlModifier)
     {
         Application::instance()->loadUrl(item->url(), Rekonq::NewTab);
@@ -1043,10 +1013,10 @@ void MainWindow::openNext(Qt::MouseButtons mouseButtons, Qt::KeyboardModifiers k
             item = new QWebHistoryItem(history->forwardItem());
         }
     }
-    
+
     if(!item)
         return;
-    
+
     if (mouseButtons == Qt::MidButton || keyboardModifiers == Qt::ControlModifier)
     {
         Application::instance()->loadUrl(item->url(), Rekonq::NewTab);
@@ -1055,7 +1025,7 @@ void MainWindow::openNext(Qt::MouseButtons mouseButtons, Qt::KeyboardModifiers k
     {
         history->goToItem(*item);
     }
-    
+
     updateActions();
 }
 
@@ -1195,7 +1165,7 @@ void MainWindow::clearPrivateData()
 
         if (clearWidget.clearDownloads->isChecked())
         {
-            Application::historyManager()->clearDownloadsHistory();
+            Application::instance()->clearDownloadsHistory();
         }
 
         if (clearWidget.clearCookies->isChecked())
@@ -1212,7 +1182,7 @@ void MainWindow::clearPrivateData()
 
         if (clearWidget.clearWebIcons->isChecked())
         {
-            QWebSettings::clearIconDatabase();
+            Application::iconManager()->clearIconCache();
         }
 
         if (clearWidget.homePageThumbs->isChecked())
@@ -1247,7 +1217,7 @@ void MainWindow::aboutToShowBackMenu()
         offset = pivot - 8;
 
     /*
-     * Need a bug report upstream. 
+     * Need a bug report upstream.
      * Seems setHtml() do some garbage in history
      * Here history->currentItem() have backItem url and currentItem (error page) title
      */
@@ -1256,18 +1226,18 @@ void MainWindow::aboutToShowBackMenu()
         QWebHistoryItem item = history->currentItem();
         KAction *action = new KAction(this);
         action->setData(listCount + offset++);
-        KIcon icon = Application::icon(item.url());
+        KIcon icon = Application::iconManager()->iconForUrl(item.url());
         action->setIcon(icon);
         action->setText(item.title());
         m_historyBackMenu->addAction(action);
     }
-         
+
     for (int i = listCount - 1; i >= 0; --i)
     {
         QWebHistoryItem item = historyList.at(i);
         KAction *action = new KAction(this);
         action->setData(i + offset);
-        KIcon icon = Application::icon(item.url());
+        KIcon icon = Application::iconManager()->iconForUrl(item.url());
         action->setIcon(icon);
         action->setText(item.title());
         m_historyBackMenu->addAction(action);
@@ -1302,7 +1272,7 @@ void MainWindow::setEncoding(QAction *qa)
         currentTab()->view()->reload();
         return;
     }
-    
+
     currentTab()->view()->settings()->setDefaultTextEncoding(currentCodec);
     currentTab()->view()->reload();
 }
@@ -1312,7 +1282,7 @@ void MainWindow::populateEncodingMenu()
 {
     QStringList codecs;
     QList<int> mibs = QTextCodec::availableMibs();
-    Q_FOREACH (const int &mib, mibs) 
+    Q_FOREACH (const int &mib, mibs)
     {
         QString codec = QLatin1String(QTextCodec::codecForMib(mib)->name());
         codecs.append(codec);
@@ -1327,13 +1297,13 @@ void MainWindow::populateEncodingMenu()
     KMenu *isciiMenu = new KMenu( QL1S("ISCII"), m_encodingMenu);
     KMenu *uniMenu = new KMenu( QL1S("Unicode"), m_encodingMenu);
     KMenu *otherMenu = new KMenu( i18n("Other"), m_encodingMenu);
-    
+
     QAction *a;
     bool isDefaultCodecUsed = true;
-    
+
     Q_FOREACH(const QString &codec, codecs)
     {
-        
+
         if( codec.startsWith( QL1S("ISO"), Qt::CaseInsensitive ) )
             a = isoMenu->addAction(codec);
         else if( codec.startsWith( QL1S("win") ) )
@@ -1342,9 +1312,9 @@ void MainWindow::populateEncodingMenu()
             a = isciiMenu->addAction(codec);
         else if( codec.startsWith( QL1S("UT") ) )
             a = uniMenu->addAction(codec);
-        else 
+        else
             a = otherMenu->addAction(codec);
-            
+
         a->setCheckable(true);
         if (currentCodec == codec)
         {
@@ -1352,11 +1322,11 @@ void MainWindow::populateEncodingMenu()
             isDefaultCodecUsed = false;
         }
     }
-    
+
     a = new QAction( i18nc("Default website encoding", "Default"), this);
     a->setCheckable(true);
     a->setChecked(isDefaultCodecUsed);
-            
+
     m_encodingMenu->addAction( a );
     m_encodingMenu->addMenu( isoMenu );
     m_encodingMenu->addMenu( winMenu );
@@ -1378,34 +1348,59 @@ bool MainWindow::queryClose()
     // this should fux bug 240432
     if(Application::instance()->sessionSaving())
         return true;
-    
-    if (m_view->count() > 1)                                                                                                              
+
+    if (m_view->count() > 1)
     {
-        int answer = KMessageBox::questionYesNoCancel(                                                                                    
-                        this,                                                                                                            
+        int answer = KMessageBox::questionYesNoCancel(
+                        this,
                         i18np("Are you sure you want to close the window?\n" "You have 1 tab open.",
-                         "Are you sure you want to close the window?\n" "You have %1 tabs open.", 
+                         "Are you sure you want to close the window?\n" "You have %1 tabs open.",
                         m_view->count()),
-                        i18n("Are you sure you want to close the window?"),                                                              
-                        KStandardGuiItem::quit(),                                                                                        
-                        KGuiItem(i18n("C&lose Current Tab"), KIcon("tab-close")),                                                        
-                        KStandardGuiItem::cancel(),                                                                                      
-                        "confirmClosingMultipleTabs"                                                                                     
-                                                        );                                                                                                                   
-                                                                                                                    
-        switch (answer)                                                                                                                   
-        {                                                                                                                                 
-            case KMessageBox::Yes:  
-                // Quit                                                                                                                         
-                return true;                                                                                                                  
-            
-            case KMessageBox::No:                                                                                                             
-                // Close only the current tab                                                                                                 
+                        i18n("Are you sure you want to close the window?"),
+                        KStandardGuiItem::quit(),
+                        KGuiItem(i18n("C&lose Current Tab"), KIcon("tab-close")),
+                        KStandardGuiItem::cancel(),
+                        "confirmClosingMultipleTabs"
+                                                        );
+
+        switch (answer)
+        {
+            case KMessageBox::Yes:
+                // Quit
+                return true;
+
+            case KMessageBox::No:
+                // Close only the current tab
                 m_view->closeTab();
-            
-            default:                                                                                                                          
-                return false;                                                                                                                 
-        }                                                                                                                                 
-    }                                                                                                                                     
-    return true;                                                                                                                          
+
+            default:
+                return false;
+        }
+    }
+    return true;
+}
+
+
+void MainWindow::saveNewToolbarConfig()
+{
+    KXmlGuiWindow::saveNewToolbarConfig();
+    setupBookmarksAndToolsShortcuts();
+}
+
+
+void MainWindow::setupBookmarksAndToolsShortcuts()
+{
+    KToolBar *mainBar = toolBar("mainToolBar");
+
+    QToolButton *bookmarksButton = qobject_cast<QToolButton*>(mainBar->widgetForAction(actionByName( QL1S("bookmarksActionMenu") )));
+    if(bookmarksButton)
+    {
+        connect(actionByName(QL1S("bookmarksActionMenu")), SIGNAL(triggered()), bookmarksButton, SLOT(showMenu()));
+    }
+
+    QToolButton *toolsButton = qobject_cast<QToolButton*>(mainBar->widgetForAction(actionByName( QL1S("rekonq_tools") )));
+    if(toolsButton)
+    {
+        connect(actionByName(QL1S("rekonq_tools")), SIGNAL(triggered()), toolsButton, SLOT(showMenu()));
+    }
 }
