@@ -35,25 +35,24 @@
 #include "rekonq_defines.h"
 
 // KDE Includes
-#include <KMainWindow>
 #include <KActionCollection>
-#include <KToolBar>
-#include <KUrl>
+#include <KXmlGuiWindow>
 
 // Forward Declarations
+class BookmarksPanel;
+class BookmarkToolBar;
 class FindBar;
 class HistoryPanel;
-class BookmarksPanel;
-class WebInspectorPanel;
-class WebTab;
 class MainView;
 class NetworkAnalyzerPanel;
+class WebInspectorPanel;
+class WebTab;
+class ZoomBar;
 
-class KAction;
+class KActionMenu;
 class KPassivePopup;
 
 class QWebFrame;
-class QSlider;
 
 
 /**
@@ -61,7 +60,7 @@ class QSlider;
  * It handles the menus, toolbars, and status bars.
  *
  */
-class REKONQ_TESTS_EXPORT MainWindow : public KMainWindow
+class REKONQ_TESTS_EXPORT MainWindow : public KXmlGuiWindow
 {
     Q_OBJECT
 
@@ -70,14 +69,16 @@ public:
     ~MainWindow();
 
     inline MainView *mainView() const { return m_view; }
-    inline KActionCollection *actionCollection() const { return m_ac; }
     inline QAction *actionByName(const QString &name) { return actionCollection()->action(name); }
-    
+
     WebTab *currentTab() const;
     virtual QSize sizeHint() const;
     void setWidgetsVisible(bool makeFullScreen);
 
+    QString selectedText() const;
+
 private:
+    void setupBookmarksAndToolsShortcuts();
     void setupActions();
     void setupTools();
     void setupToolbars();
@@ -100,13 +101,21 @@ public slots:
 
     void updateActions();
 
-    void setZoomSliderFactor(qreal factor);
+    virtual void configureToolbars();
+
+    // Find Bar slots
+    void findNext();
+    void findPrevious();
+    void updateHighlight();
 
 signals:
     // switching tabs
     void ctrlTabPressed();
     void shiftCtrlTabPressed();
 
+protected slots:
+    void saveNewToolbarConfig();
+
 protected:
     /**
     * Filters (SHIFT + ) CTRL + TAB events and emit (shift)ctrlTabPressed()
@@ -116,7 +125,7 @@ protected:
     void keyPressEvent(QKeyEvent *event);
 
     bool queryClose();
-    
+
 private slots:
     void postLaunch();
     void browserLoading(bool);
@@ -129,15 +138,6 @@ private slots:
     // Find Action slots
     void find(const QString &);
     void matchCaseUpdate();
-    void findNext();
-    void findPrevious();
-    void highlightAll();
-
-    // Zoom slots
-    void zoomIn();
-    void zoomNormal();
-    void zoomOut();
-    void setZoomFactor(int factor);
 
     // File Menu slots
     void openLocation();
@@ -164,31 +164,32 @@ private slots:
     void populateEncodingMenu();
 
     void enableNetworkAnalysis(bool);
-    
+
+    void initBookmarkBar();
+    void initToolsMenu();
+
 private:
     MainView *m_view;
     FindBar *m_findBar;
+    ZoomBar *m_zoomBar;
 
     HistoryPanel *m_historyPanel;
     BookmarksPanel *m_bookmarksPanel;
     WebInspectorPanel *m_webInspectorPanel;
     NetworkAnalyzerPanel *m_analyzerPanel;
-    
+
     KAction *m_stopReloadAction;
     KMenu *m_historyBackMenu;
     KMenu *m_encodingMenu;
 
-    KToolBar *m_mainBar;
-    KToolBar *m_bmBar;
-
-    QSlider *m_zoomSlider;
+    BookmarkToolBar *m_bookmarksBar;
 
     QString m_lastSearch;
 
     KPassivePopup *m_popup;
     QTimer *m_hidePopup;
 
-    KActionCollection *m_ac;
+    KActionMenu *m_toolsMenu;
 };
 
 #endif // MAINWINDOW_H
