@@ -31,17 +31,14 @@
 #include "networkaccessmanager.moc"
 
 // Local Includes
-#include "application.h"
 #include "adblockmanager.h"
+#include "application.h"
 #include "webpage.h"
 
 // KDE Includes
 #include <KLocale>
 #include <KProtocolManager>
 
-// Qt Includes
-#include <QtNetwork/QNetworkReply>
-
 
 NetworkAccessManager::NetworkAccessManager(QObject *parent)
         : AccessManager(parent)
@@ -88,17 +85,51 @@ QNetworkReply *NetworkAccessManager::createRequest(QNetworkAccessManager::Operat
         break;
     }
 
+    // WARNING
+    // There are actually 2 exceptions here handled with QNAM
+    // instead of KIO that need fixes upstream before removing. They are:
+    // 1) DeleteOperation
+    // 2) CustomOperation
 
-    if (op == QNetworkAccessManager::GetOperation)
+    switch(op)
     {
+    case QNetworkAccessManager::HeadOperation:
+        break;
+
+    case QNetworkAccessManager::GetOperation:
         reply = Application::adblockManager()->block(req, parentPage);
+        break;
+
+    case QNetworkAccessManager::PutOperation:
+        break;
+
+    case QNetworkAccessManager::PostOperation:
+        break;
+
+    case QNetworkAccessManager::DeleteOperation:
+        kDebug() << "DELETE OPERATION...";
+        reply = QNetworkAccessManager::createRequest(op, req, outgoingData);
+        if(!reply)
+            kDebug() << "OOOOOOOOOOOOOOOOOOO DELETE REPLY NULL";
+        break;
+
+    case QNetworkAccessManager::CustomOperation:
+        kDebug() << "CUSTOM OPERATION...";
+        reply = QNetworkAccessManager::createRequest(op, req, outgoingData);
+        if(!reply)
+            kDebug() << "OOOOOOOOOOOOOOOOOOO CUSTOM REPLY NULL";
+        break;
+
+    default:
+        kDebug() << "NON EXTANT CASE...";
+        break;
     }
 
     if(!reply)
         reply = AccessManager::createRequest(op, req, outgoingData);
 
     if(parentPage && parentPage->hasNetworkAnalyzerEnabled())
-        emit networkData( op, req, reply ); 
+        emit networkData( op, req, reply );
 
     return reply;
 }
