@@ -31,36 +31,28 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "newtabpage.h"
-#include "application.h"
-#include "mainwindow.h"
-#include "mainview.h"
-#include "urlbar.h"
-#include "webtab.h"
-#include "historymanager.h"
 #include "adblockmanager.h"
+#include "application.h"
+#include "historymanager.h"
+#include "mainview.h"
+#include "mainwindow.h"
+#include "newtabpage.h"
+#include "urlbar.h"
+#include "webpage.h"
+#include "webtab.h"
 
 // KDE Includes
-#include <klocalizedstring.h>
-#include <KUrl>
-#include <KRun>
-#include <KToolInvocation>
-#include <KStandardDirs>
-#include <KMimeType>
-#include <KIconLoader>
+#include <KIO/Job>
 #include <KDirLister>
-#include <KFileItem>
-#include <KJob>
-#include <kio/udsentry.h>
+#include <KLocalizedString>
 #include <KMessageBox>
+#include <KProcess>
+#include <KStandardDirs>
+#include <KToolInvocation>
 
 // Qt Includes
-#include <QLatin1String>
-#include <QNetworkRequest>
-#include <QWebFrame>
-#include <QDir>
-#include <QFile>
-#include <QDateTime>
+#include <QtNetwork/QNetworkRequest>
+#include <QtWebKit/QWebFrame>
 
 
 ProtocolHandler::ProtocolHandler(QObject *parent)
@@ -106,7 +98,8 @@ bool ProtocolHandler::preHandling(const QNetworkRequest &request, QWebFrame *fra
             if(scriptSource.isEmpty())
                 return false;
         }
-        
+
+        kDebug() << "EVALUATING JAVASCRIPT...";
         QVariant result = frame->evaluateJavaScript(scriptSource);
         return true;
     }
@@ -118,6 +111,22 @@ bool ProtocolHandler::preHandling(const QNetworkRequest &request, QWebFrame *fra
         return true;
     }
 
+    // "apturl" handling
+    if (_url.protocol() == QL1S ("apt"))
+    {
+      //Declare apturl as QString
+      QString apturl="apturl";
+      //We need to convert the url to QStringList to pass as a argument to apturl
+      QStringList host;
+      host << _url.url();
+
+      if ( KProcess::execute (apturl,host)==0)
+        return true;
+      else
+        return false;
+
+    }
+
     // "abp" handling
     if (_url.protocol() == QL1S("abp"))
     {
@@ -128,13 +137,14 @@ bool ProtocolHandler::preHandling(const QNetworkRequest &request, QWebFrame *fra
     // "about" handling
     if (_url.protocol() == QL1S("about"))
     {
+        QByteArray encodedUrl = _url.toEncoded();
         // let webkit manage the about:blank url...
-        if (_url == KUrl("about:blank"))
+        if (encodedUrl == QByteArray("about:blank"))
         {
             return false;
         }
 
-        if (_url == KUrl("about:home"))
+        if (encodedUrl == QByteArray("about:home"))
         {
             switch (ReKonfig::newTabStartPage())
             {
