@@ -197,9 +197,7 @@ void SettingsDialog::saveSettings()
     d->shortcutsEditor->save();
     d->ebrowsingModule->save();
     
-    SearchEngine::loadDefaultWS();
-    SearchEngine::loadDelimiter();
-    SearchEngine::loadFavorites();
+    SearchEngine::reload();
 
     updateButtons();
     emit settingsChanged("ReKonfig");
@@ -219,3 +217,16 @@ bool SettingsDialog::hasChanged()
            || d->shortcutsEditor->isModified();
     ;
 }
+
+
+bool SettingsDialog::isDefault()
+{
+    bool isDef = KConfigDialog::isDefault();
+    
+    if(isDef)
+    {
+        // check our private widget values
+        isDef = d->appearanceWidg->isDefault();
+    }
+    return isDef;
+}
