@@ -50,6 +50,9 @@ public:
 
     virtual bool hasChanged();
 
+protected:
+    virtual bool isDefault();
+    
 private:
     Private* const d;
 
