@@ -33,29 +33,26 @@
 
 // Local Includes
 #include "rekonq.h"
+
 #include "application.h"
+#include "iconmanager.h"
+#include "mainview.h"
 #include "mainwindow.h"
-#include "urlbar.h"
+#include "webpage.h"
 #include "webtab.h"
 #include "websnap.h"
-#include "mainview.h"
 
 // KDE Includes
-#include <KShortcut>
-#include <KStandardShortcut>
-#include <KGlobalSettings>
-#include <KPassivePopup>
-#include <KMenu>
 #include <KActionMenu>
+#include <KMenu>
+#include <KPassivePopup>
+#include <KToolBar>
 
 // Qt Includes
-#include <QString>
-#include <QFont>
-#include <QToolButton>
-#include <QLabel>
-#include <QMouseEvent>
-#include <QPainter>
-#include <QVBoxLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QLayout>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QToolButton>
 
 
 #define BASE_WIDTH_DIVISOR    4
@@ -171,7 +168,7 @@ void TabBar::showTabPreview()
     // no previews during load
     if (indexedTab->progress() != 0)
         return;
-    
+
     int w = tabSizeHint(m_currentTabPreviewIndex).width();
     int h = w * ((0.0 + currentTab->height()) / currentTab->width());
 
@@ -207,7 +204,7 @@ void TabBar::mouseMoveEvent(QMouseEvent *event)
     }
 
     KTabBar::mouseMoveEvent(event);
-    
+
     if (ReKonfig::alwaysShowTabPreviews())
     {
         //Find the tab under the mouse
@@ -286,7 +283,7 @@ void TabBar::mousePressEvent(QMouseEvent *event)
 void TabBar::contextMenu(int tab, const QPoint &pos)
 {
     setupHistoryActions();
-    
+
     m_actualIndex = tab;
 
     KMenu menu;
@@ -312,7 +309,7 @@ void TabBar::contextMenu(int tab, const QPoint &pos)
 void TabBar::emptyAreaContextMenu(const QPoint &pos)
 {
     setupHistoryActions();
-    
+
     KMenu menu;
     MainWindow *mainWindow = Application::instance()->mainWindow();
 
@@ -322,6 +319,12 @@ void TabBar::emptyAreaContextMenu(const QPoint &pos)
     menu.addSeparator();
     menu.addAction(mainWindow->actionByName( QL1S("reload_all_tabs") ));
 
+    KToolBar *mainBar = mainWindow->toolBar("mainToolBar");
+    if(!mainBar->isVisible())
+    {
+        menu.addAction( mainBar->toggleViewAction() );
+    }
+
     menu.exec(pos);
 }
 
@@ -353,7 +356,7 @@ void TabBar::setupHistoryActions()
 {
     MainWindow *w = Application::instance()->mainWindow();
     MainView *mv = qobject_cast<MainView *>(parent());
-    
+
     QAction *openClosedTabsAction = w->actionByName( QL1S("open_closed_tabs") );
     openClosedTabsAction->setEnabled( mv->recentlyClosedTabs().size() > 0 );
 
@@ -373,7 +376,7 @@ void TabBar::setupHistoryActions()
 
     foreach (const HistoryItem &item, mv->recentlyClosedTabs())
     {
-        KAction *a = new KAction(Application::icon(item.url), item.title, this);
+        KAction *a = new KAction(Application::iconManager()->iconForUrl(item.url), item.title, this);
         a->setData(item.url);
         connect(a, SIGNAL(triggered()), mv, SLOT(openClosedTab()));
         am->addAction(a);
