@@ -54,8 +54,8 @@
 CompletionWidget::CompletionWidget(QWidget *parent)
         : QFrame(parent, Qt::ToolTip)
         , _parent(parent)
-        , _currentIndex(-1)
-        , _searchEngine(SearchEngine::defaultEngine())
+        , _currentIndex(0)
+        , _hasSuggestions(false)
 {
     setFrameStyle(QFrame::Panel);
     setLayoutDirection(Qt::LeftToRight);
@@ -70,7 +70,7 @@ void CompletionWidget::insertSearchList(const UrlSearchList &list, const QString
 {
     if (!isVisible())
     {
-        _searchEngine = SearchEngine::defaultEngine();
+        UrlResolver::setSearchEngine(SearchEngine::defaultEngine());
     }
 
     _list = list;
@@ -79,19 +79,60 @@ void CompletionWidget::insertSearchList(const UrlSearchList &list, const QString
     {
         ListItem *suggestion = ListItemFactory::create(item, text, this);
         suggestion->setBackgroundRole(i % 2 ? QPalette::AlternateBase : QPalette::Base);
-        connect(suggestion, SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)), this, SLOT(itemChosen(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)));
+        connect(suggestion, 
+                SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)), 
+                this, 
+                SLOT(itemChosen(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)));
         connect(this, SIGNAL(nextItemSubChoice()), suggestion, SLOT(nextItemSubChoice()));
+        
         suggestion->setObjectName(QString::number(i++));
         layout()->addWidget(suggestion);
     }
 }
 
 
+void CompletionWidget::updateSearchList(const UrlSearchList &list, const QString& text)
+{
+    static int counter = 0;
+    counter++;
+    kDebug() << counter;
+    if(_hasSuggestions || _typedString != text)
+        return;
+    _hasSuggestions = true;
+    
+    if (_resList.count() > 0)
+    {
+        clear();
+        insertSearchList(_resList, text);
+    
+        UrlSearchList sugList = list.mid(0,4);
+
+        // add new suggestions to the list
+        int offset = _list.count();
+        Q_FOREACH(const UrlSearchItem &item, sugList)
+        {
+            ListItem *suggestion = ListItemFactory::create(item, text, this);
+            suggestion->setBackgroundRole(offset % 2 ? QPalette::AlternateBase : QPalette::Base);
+            connect(suggestion, 
+                    SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)), 
+                    this, 
+                    SLOT(itemChosen(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)));
+            connect(this, SIGNAL(nextItemSubChoice()), suggestion, SLOT(nextItemSubChoice()));
+            
+            suggestion->setObjectName(QString::number(offset++));
+            layout()->addWidget(suggestion);
+        }
+        _list.append(sugList);
+        popup();
+    }
+}
+
+
 void CompletionWidget::sizeAndPosition()
 {
     setFixedWidth(_parent->width());
 
-    int h=0;
+    int h = 0;
     for (int i = 0; i < layout()->count(); i++)
     {
         QWidget *widget = layout()->itemAt(i)->widget();
@@ -107,7 +148,7 @@ void CompletionWidget::sizeAndPosition()
 
 void CompletionWidget::popup()
 {
-    down();
+    findChild<ListItem *>(QString::number(0))->activate(); //activate first listitem
     sizeAndPosition();
     if (!isVisible())
         show();
@@ -117,40 +158,44 @@ void CompletionWidget::popup()
 void CompletionWidget::up()
 {
     // deactivate previous
-    if (_currentIndex != -1)
-    {
-        ListItem *widget = findChild<ListItem *>(QString::number(_currentIndex));
-        widget->deactivate();
-    }
+    findChild<ListItem *>(QString::number(_currentIndex))->deactivate(); // deactivate previous
 
     if (_currentIndex > 0)
         _currentIndex--;
     else
         _currentIndex = layout()->count() - 1;
 
-    // activate "new" current
-    ListItem *widget = findChild<ListItem *>(QString::number(_currentIndex));
-    widget->activate();
+    activateCurrentListItem();
 }
 
 
 void CompletionWidget::down()
 {
-    // deactivate previous
-    if (_currentIndex != -1)
-    {
-        ListItem *widget = findChild<ListItem *>(QString::number(_currentIndex));
-        widget->deactivate();
-    }
+    findChild<ListItem *>(QString::number(_currentIndex))->deactivate(); // deactivate previous
 
     if (_currentIndex < _list.count() - 1)
         _currentIndex++;
     else
         _currentIndex = 0;
 
+    activateCurrentListItem();
+}
+
+
+void CompletionWidget::activateCurrentListItem()
+{
+    UrlBar *bar = qobject_cast<UrlBar *>(_parent);
+
     // activate "new" current
     ListItem *widget = findChild<ListItem *>(QString::number(_currentIndex));
     widget->activate();
+
+    //update text of the url bar
+    bar->blockSignals(true); //without compute suggestions
+    bar->setQUrl(widget->text());
+    bar->blockSignals(false);
+    bar->setFocus();
+    bar->setCursorPosition(bar->text().length());
 }
 
 
@@ -162,27 +207,27 @@ void CompletionWidget::clear()
         delete child->widget();
         delete child;
     }
-    _currentIndex = -1;
+    _currentIndex = 0;
+    _hasSuggestions = false;
 }
 
 
-bool CompletionWidget::eventFilter(QObject *o, QEvent *e)
+bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
 {
-    int type = e->type();
-    QWidget *wid = qobject_cast<QWidget*>(o);
+    int type = ev->type();
+    QWidget *wid = qobject_cast<QWidget*>(obj);
 
-    if (o == this)
+    if (obj == this)
     {
         return false;
     }
 
-    //hide conditions of the CompletionWidget
+    // hide conditions of the CompletionWidget
     if (wid
-            && ((wid == _parent && (type == QEvent::Move || type == QEvent::Resize))
-                || ((wid->windowFlags() & Qt::Window)
-                    && (type == QEvent::Move || type == QEvent::Hide || type == QEvent::WindowDeactivate)
-                    && wid == _parent->window())
-                || (type == QEvent::MouseButtonPress && !isAncestorOf(wid)))
+        && ((wid == _parent 
+            && (type == QEvent::Move || type == QEvent::Resize)) || ((wid->windowFlags() & Qt::Window)
+            && (type == QEvent::Move || type == QEvent::Hide || type == QEvent::WindowDeactivate)
+            && wid == _parent->window()) || (type == QEvent::MouseButtonPress && !isAncestorOf(wid)))
        )
     {
         hide();
@@ -197,49 +242,84 @@ bool CompletionWidget::eventFilter(QObject *o, QEvent *e)
         
         if (type == QEvent::KeyPress)
         {
-            QKeyEvent *ev = static_cast<QKeyEvent *>(e);
-            switch (ev->key())
+            QKeyEvent *kev = static_cast<QKeyEvent *>(ev);
+            switch (kev->key())
             {
             case Qt::Key_Up:
             case Qt::Key_Backtab:
-                if (ev->modifiers() == Qt::NoButton || (ev->modifiers() & Qt::ShiftModifier))
+                if (kev->modifiers() == Qt::NoButton || (kev->modifiers() & Qt::ShiftModifier))
                 {
                     up();
-                    ev->accept();
+                    kev->accept();
                     return true;
                 }
                 break;
 
             case Qt::Key_Down:
             case Qt::Key_Tab:
-                if (ev->modifiers() == Qt::NoButton)
+                if (kev->modifiers() == Qt::NoButton)
                 {
                     down();
-                    ev->accept();
+                    kev->accept();
                     return true;
                 }
-                if (ev->modifiers() & Qt::ControlModifier)
+                if (kev->modifiers() & Qt::ControlModifier)
                 {
                     emit nextItemSubChoice();
-                    ev->accept();
+                    kev->accept();
                     return true;
                 }
                 break;
 
             case Qt::Key_Enter:
             case Qt::Key_Return:
-                w = qobject_cast<UrlBar *>(parent());               
-                if(w->text() == _typedString)
+                w = qobject_cast<UrlBar *>(parent());
+
+                if (!w->text().startsWith(QL1S("http://"), Qt::CaseInsensitive))
                 {
-                    child = findChild<ListItem *>(QString::number(_currentIndex));
+                    QString append;
+                    if (kev->modifiers() == Qt::ControlModifier)
+                    {
+                        append = QL1S(".com");
+                    }
+                    else if (kev->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))
+                    {
+                        append = QL1S(".org");
+                    }
+                    else if (kev->modifiers() == Qt::ShiftModifier)
+                    {
+                        append = QL1S(".net");
+                    }
+
+                    if (!append.isEmpty())
+                    {
+                        QUrl url(QL1S("http://www.") + w->text());
+                        QString host = url.host();
+                        if (!host.endsWith(append, Qt::CaseInsensitive))
+                        {
+                            host += append;
+                            url.setHost(host);
+                        }
+                        
+                        emit chosenUrl(url, Rekonq::CurrentTab);
+                    }
+                }
+
+                
+                if( _currentIndex == -1)
+                    _currentIndex = 0;
+                child = findChild<ListItem *>(QString::number(_currentIndex));
+                if(child && _currentIndex!=0) //the completionwidget is visible and the user had press down
+                {
+                    //we can use the url of the listitem
                     emit chosenUrl(child->url(), Rekonq::CurrentTab);
                 }
-                else
+                else //the user type too fast (completionwidget not visible or suggestion not downloaded)
                 {
-                    // this will be used just on fast typing..
-                    emit chosenUrl(KUrl(w->text()), Rekonq::CurrentTab);
+                    UrlResolver res(w->text());
+                    emit chosenUrl(res.orderedSearchItems().first().url, Rekonq::CurrentTab);
                 }
-                ev->accept();
+                kev->accept();
                 hide();
                 return true;
                 
@@ -250,7 +330,7 @@ bool CompletionWidget::eventFilter(QObject *o, QEvent *e)
         }
     }
 
-    return QFrame::eventFilter(o, e);
+    return QFrame::eventFilter(obj, ev);
 }
 
 
@@ -272,10 +352,15 @@ void CompletionWidget::setVisible(bool visible)
 
 void CompletionWidget::itemChosen(ListItem *item, Qt::MouseButton button, Qt::KeyboardModifiers modifier)
 {
-    if (button == Qt::MidButton || modifier == Qt::ControlModifier)
+    if (button == Qt::MidButton 
+        || modifier == Qt::ControlModifier)
+    {
         emit chosenUrl(item->url(), Rekonq::NewFocusedTab);
+    }
     else
+    {
         emit chosenUrl(item->url(), Rekonq::CurrentTab);
+    }
     hide();
 }
 
@@ -294,12 +379,8 @@ void CompletionWidget::suggestUrls(const QString &text)
         return;
     }
 
-    UrlResolver res(text);
-    UrlSearchList list = res.orderedSearchItems();
-    if (list.count() > 0)
-    {
-        clear();
-        insertSearchList(list, text);
-        popup();
-    }
+    UrlResolver *res = new UrlResolver(text);
+    connect(res, SIGNAL(suggestionsReady(const UrlSearchList &, const QString &)), this, SLOT(updateSearchList(const UrlSearchList &, const QString &)));
+    res->computeSuggestions();
+    _resList = res->orderedSearchItems();
 }
