@@ -37,6 +37,7 @@
 #include "websnap.h"
 #include "completionwidget.h"
 #include "searchengine.h"
+#include "iconmanager.h"
 
 // KDE Includes
 #include <KIcon>
@@ -139,6 +140,12 @@ KUrl ListItem::url()
 }
 
 
+QString ListItem::text()
+{
+    return m_url.url();
+}
+
+
 void ListItem::nextItemSubChoice()
 {
     //will be override
@@ -161,6 +168,7 @@ TypeIconLabel::TypeIconLabel(int type, QWidget *parent)
     if (type & UrlSearchItem::Browse) hLayout->addWidget(getIcon("applications-internet"));
     if (type & UrlSearchItem::Bookmark) hLayout->addWidget(getIcon("rating"));
     if (type & UrlSearchItem::History) hLayout->addWidget(getIcon("view-history"));
+    if (type & UrlSearchItem::Suggestion) hLayout->addWidget(getIcon("help-hint"));
 }
 
 
@@ -180,12 +188,20 @@ QLabel *TypeIconLabel::getIcon(QString icon)
 IconLabel::IconLabel(const QString &icon, QWidget *parent)
         : QLabel(parent)
 {
-    QPixmap pixmapIcon = Application::icon(KUrl(icon)).pixmap(16);
+    QPixmap pixmapIcon = Application::iconManager()->iconForUrl(KUrl(icon)).pixmap(16);
     setFixedSize(16, 16);
     setPixmap(pixmapIcon);
 }
 
 
+IconLabel::IconLabel(const KIcon &icon, QWidget *parent)
+    : QLabel(parent)
+{
+    QPixmap pixmapIcon = icon.pixmap(16);
+    setFixedSize(16, 16);
+    setPixmap(pixmapIcon);
+}
+
 // ---------------------------------------------------------------
 
 
@@ -193,14 +209,35 @@ TextLabel::TextLabel(const QString &text, const QString &textToPointOut, QWidget
         : QLabel(parent)
 {
     QString t = text;
-    if (!textToPointOut.isEmpty())
-        t = t.replace(QRegExp('(' + textToPointOut + ')', Qt::CaseInsensitive), "<b>\\1</b>");
+    const bool wasItalic = t.startsWith(QL1S("<i>"));
+    if (wasItalic)
+        t.remove(QRegExp("<[/ib]*>"));
+
+    t = Qt::escape(t);
+    QString ss = Qt::escape(textToPointOut);
+    t.replace(QRegExp('(' + ss + ')', Qt::CaseInsensitive), "<b>\\1</b>");
+
+    if (wasItalic)
+        t = QL1S("<i>") + t + QL1S("</i>");
 
     setText(t);
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
 }
 
 
+TextLabel::TextLabel(QWidget *parent)
+    : QLabel(parent)
+{
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);    
+}
+
+
+void TextLabel::setEngineText(const QString &engine, const QString &text)
+{
+    setText( i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for <b>%2</b>", engine, Qt::escape(text) ) );
+}
+
+
 //--------------------------------------------------------------------------------------------
 
 
@@ -265,22 +302,11 @@ SearchListItem::SearchListItem(const UrlSearchItem &item, const QString &text, Q
         : ListItem(item, parent)
         , m_text(text)
 {
-    QString query = text;
-    KService::Ptr engine = SearchEngine::fromString(text);
-    if (engine)
-    {
-        query = query.remove(0, text.indexOf(SearchEngine::delimiter()) + 1);
-    }
-    else
-    {
-        engine = qobject_cast<CompletionWidget *>(parent)->searchEngine();
-    }
-
-    m_url = SearchEngine::buildQuery(engine, query);
-
-    m_iconLabel = new IconLabel("edit-find", this); //TODO: get the default engine icon (will be easy in KDE SC 4.5)
-    m_titleLabel = new TextLabel(searchItemTitle(engine->name(), query), QString(), this);
-    m_engineBar = new EngineBar(engine, parent);
+    m_iconLabel = new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this);
+    QString query = SearchEngine::extractQuery(text);
+    m_titleLabel = new TextLabel(this);
+    m_titleLabel->setEngineText(item.title, query);
+    m_engineBar = new EngineBar(UrlResolver::searchEngine(), parent);
 
     QHBoxLayout *hLayout = new QHBoxLayout;
     hLayout->setSpacing(4);
@@ -297,18 +323,24 @@ SearchListItem::SearchListItem(const UrlSearchItem &item, const QString &text, Q
 }
 
 
-QString SearchListItem::searchItemTitle(QString engine, QString text)
+QString SearchListItem::text()
 {
-    return QString(i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for <b>%2</b>", engine, Qt::escape(text)));
+    return m_text;
 }
 
 
+// QString SearchListItem::searchItemTitle(QString engine, QString text)
+// {
+//     return QString(i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for %2", engine, text));
+// }
+
+
 void SearchListItem::changeSearchEngine(KService::Ptr engine)
 {
-    m_titleLabel->setText(searchItemTitle(engine->name(), m_text));
-    m_iconLabel->setPixmap(Application::icon(KUrl(engine->property("Query").toString())).pixmap(16));
+    m_titleLabel->setEngineText(engine->name(), m_text);
+    m_iconLabel->setPixmap( Application::iconManager()->iconForUrl(KUrl(engine->property("Query").toString())).pixmap(16) );
     m_url = SearchEngine::buildQuery(engine, m_text);
-    qobject_cast<CompletionWidget *>(parent())->setSearchEngine(engine);
+    UrlResolver::setSearchEngine(engine);
 }
 
 
@@ -349,7 +381,7 @@ KAction *EngineBar::newEngineAction(KService::Ptr engine, KService::Ptr selected
     KUrl url = KUrl( u.toString( QUrl::RemovePath | QUrl::RemoveQuery ) );
     
     kDebug() << "Engine NAME: " << engine->name() << " URL: " << url;
-    KAction *a = new KAction(Application::icon(url), engine->name(), this);
+    KAction *a = new KAction(Application::iconManager()->iconForUrl(url), engine->name(), this);
     a->setCheckable(true);
     if (engine->desktopEntryName() == selectedEngine->desktopEntryName()) a->setChecked(true);
     a->setData(engine->entryPath());
@@ -390,6 +422,29 @@ void EngineBar::selectNextEngine()
 // ---------------------------------------------------------------
 
 
+SuggestionListItem::SuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+        : ListItem(item, parent)
+        , m_text(item.title)
+{
+    QHBoxLayout *hLayout = new QHBoxLayout;
+    hLayout->setSpacing(4);
+
+    hLayout->addWidget(new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this));
+    hLayout->addWidget(new TextLabel(item.title, SearchEngine::extractQuery(text), this));
+    hLayout->addWidget(new TypeIconLabel(item.type, this));
+
+    setLayout(hLayout);
+}
+
+
+QString SuggestionListItem::text()
+{
+    return m_text;
+}
+
+// ---------------------------------------------------------------
+
+
 BrowseListItem::BrowseListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
         : ListItem(item, parent)
 {
@@ -427,7 +482,15 @@ ListItem *ListItemFactory::create(const UrlSearchItem &item, const QString &text
         }
         else
         {
-            newItem = new PreviewListItem(item, text, parent);
+
+            if (item.type & UrlSearchItem::Suggestion)
+            {
+                newItem = new SuggestionListItem(item, text, parent);
+            }
+            else
+            {
+                newItem = new PreviewListItem(item, text, parent);
+            }
         }
     }
 
