@@ -41,9 +41,14 @@
 #include "webpage.h"
 #include "webview.h"
 #include "completionwidget.h"
+#include "bookmarkprovider.h"
+#include "bookmarkowner.h"
+#include "bookmarkwidget.h"
+#include "iconmanager.h"
 
 // KDE Includes
 #include <KCompletionBox>
+#include <KStandardDirs>
 
 // Qt Includes
 #include <QtGui/QPainter>
@@ -84,9 +89,6 @@ UrlBar::UrlBar(QWidget *parent)
     // doesn't show the clear button
     setClearButtonShown(false);
 
-    // trap Key_Enter & Key_Return events, while emitting the returnPressed signal
-    setTrapReturnKey(true);
-
     // insert decoded URLs
     setUrlDropsEnabled(true);
 
@@ -102,17 +104,18 @@ UrlBar::UrlBar(QWidget *parent)
     _tab = qobject_cast<WebTab *>(parent);
 
     connect(_tab, SIGNAL(loadProgressing()), this, SLOT(update()));
-    
+
     connect(_tab->view(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(setQUrl(const QUrl &)));
     connect(_tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
     connect(_tab->view(), SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
+    connect(_tab->view(), SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
 
-    // load typed urls
-    connect(this, SIGNAL(returnPressed(const QString &)), this, SLOT(loadTyped(const QString &)));
+    // bookmark icon
+    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(const QString &, const QString &)), this, SLOT(onBookmarksChanged()));
 
     _suggestionTimer->setSingleShot(true);
     connect(_suggestionTimer, SIGNAL(timeout()), this, SLOT(suggest()));
-    
+
     activateSuggestions(true);
 }
 
@@ -129,7 +132,6 @@ void UrlBar::setQUrl(const QUrl& url)
 {
     if (url.scheme() == QL1S("about"))
     {
-        _icon->setIcon(KIcon("arrow-right"));
         clear();
         setFocus();
     }
@@ -138,7 +140,7 @@ void UrlBar::setQUrl(const QUrl& url)
         clearFocus();
         KLineEdit::setUrl(url);
         setCursorPosition(0);
-        _icon->setIcon(Application::icon(url));
+        refreshFavicon();
     }
 }
 
@@ -154,10 +156,10 @@ void UrlBar::activated(const KUrl& url, Rekonq::OpenType type)
 
 
 void UrlBar::paintEvent(QPaintEvent *event)
-{   
+{
     QColor backgroundColor;
     QColor foregroundColor;
-    
+
     if (_privateMode)
     {
         backgroundColor = QColor(220, 220, 220);  // light gray
@@ -185,9 +187,23 @@ void UrlBar::paintEvent(QPaintEvent *event)
     }
     else
     {
-        QColor loadingColor = Application::palette().color(QPalette::ToolTipBase);
+        QColor highlight = Application::palette().color(QPalette::Highlight);
 
-        QLinearGradient gradient( QPoint(0, 0), QPoint(width(), height()) );
+        int r = (highlight.red()+2*backgroundColor.red())/3;
+        int g = (highlight.green()+2*backgroundColor.green())/3;
+        int b = (highlight.blue()+2*backgroundColor.blue())/3;
+
+        QColor loadingColor(r, g, b);
+
+        if (abs(loadingColor.lightness() - backgroundColor.lightness()) < 20) //eg. Gaia color scheme
+        {
+            r = (2*highlight.red()+backgroundColor.red())/3;
+            g = (2*highlight.green()+backgroundColor.green())/3;
+            b = (2*highlight.blue()+backgroundColor.blue())/3;
+            loadingColor = QColor(r, g, b);
+        }
+
+        QLinearGradient gradient( QPoint(0, 0), QPoint(width(), 0) );
         gradient.setColorAt(0, loadingColor);
         gradient.setColorAt(((double)progr) / 100 - .000001, loadingColor);
         gradient.setColorAt(((double)progr) / 100, backgroundColor);
@@ -218,7 +234,8 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
     // this handles the Modifiers + Return key combinations
     QString currentText = text().trimmed();
     if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
-            && !currentText.startsWith(QL1S("http://"), Qt::CaseInsensitive))
+            && !currentText.startsWith(QL1S("http://"), Qt::CaseInsensitive)
+            && event->modifiers() != Qt::NoModifier)
     {
         QString append;
         if (event->modifiers() == Qt::ControlModifier)
@@ -234,17 +251,28 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
             append = QL1S(".net");
         }
 
-        QUrl url(QL1S("http://www.") + currentText);
-        QString host = url.host();
-        if (!host.endsWith(append, Qt::CaseInsensitive))
+        if (!append.isEmpty())
         {
-            host += append;
-            url.setHost(host);
-            setText(url.toString());
+            QUrl url(QL1S("http://www.") + currentText);
+            QString host = url.host();
+            if (!host.endsWith(append, Qt::CaseInsensitive))
+            {
+                host += append;
+                url.setHost(host);
+            }
+
+            // now, load it!
+            activated(url);
         }
     }
 
-    if (event->key() == Qt::Key_Escape)
+    else if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
+                && !currentText.isEmpty())
+    {
+        loadTyped(currentText);
+    }
+
+    else if (event->key() == Qt::Key_Escape)
     {
         clearFocus();
         event->accept();
@@ -286,8 +314,12 @@ void UrlBar::loadFinished()
         return;
     }
 
+    // show bookmark info
+    IconButton *bt = addRightIcon(UrlBar::BK);
+    connect(bt, SIGNAL(clicked(const QPoint &)), this, SLOT(showBookmarkInfo(const QPoint &)));
+
     // show KGet downloads??
-    if (ReKonfig::kgetList())
+    if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
     {
         IconButton *bt = addRightIcon(UrlBar::KGet);
         connect(bt, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(downloadAllContentsWithKGet(QPoint)));
@@ -308,7 +340,7 @@ void UrlBar::loadFinished()
     }
 
     // we need to update urlbar after the right icon settings
-    // removing this code (where setStyleSheet automatically calls update) needs adding again 
+    // removing this code (where setStyleSheet automatically calls update) needs adding again
     // an update call
     kDebug() << "resetting stylesheet";
     int rightIconWidth = 25 * (_rightIconsList.count());
@@ -316,6 +348,36 @@ void UrlBar::loadFinished()
 }
 
 
+void UrlBar::showBookmarkInfo(const QPoint &pos)
+{
+    if( _tab->url().scheme() == QL1S("about") )
+        return;
+
+    KBookmark bookmark = Application::bookmarkProvider()->bookmarkForUrl(_tab->url());
+
+    IconButton *bt = qobject_cast<IconButton *>(this->sender());
+    if (!bt)
+        return;
+
+    if (bookmark.isNull())
+    {
+        bookmark = Application::bookmarkProvider()->bookmarkOwner()->bookmarkCurrentPage();
+    }
+    else
+    {
+        BookmarkWidget *widget = new BookmarkWidget(bookmark, window());
+        widget->showAt(pos);
+    }
+}
+
+
+void UrlBar::onBookmarksChanged()
+{
+    clearRightIcons();
+    loadFinished();
+}
+
+
 void UrlBar::loadTyped(const QString &text)
 {
     activated( KUrl(text) );
@@ -369,6 +431,18 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setIcon(KIcon("object-locked"));
         rightIcon->setToolTip(i18n("Show SSL Info"));
         break;
+    case UrlBar::BK:
+        if (Application::bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
+        {
+            rightIcon->setIcon(KIcon("bookmarks").pixmap(32,32, QIcon::Disabled));
+            rightIcon->setToolTip(i18n("Bookmark this page"));
+        }
+        else
+        {
+            rightIcon->setIcon(KIcon("bookmarks"));
+            rightIcon->setToolTip(i18n("Edit this bookmark"));
+        }
+        break;
     default:
         kDebug() << "ERROR.. default non extant case!!";
         break;
@@ -416,7 +490,7 @@ void UrlBar::detectTypedString(const QString &typed)
         QTimer::singleShot(0, this, SLOT(suggest()));
         return;
     }
-    
+
     if(_suggestionTimer->isActive())
         _suggestionTimer->stop();
     _suggestionTimer->start(50);
@@ -428,3 +502,14 @@ void UrlBar::suggest()
     if(!_box.isNull())
         _box.data()->suggestUrls( text() );
 }
+
+
+void UrlBar::refreshFavicon()
+{
+    KUrl u = _tab->url();
+    if(u.scheme() == QL1S("about")) {
+        _icon->setIcon(KIcon("arrow-right"));
+        return;
+    }
+    _icon->setIcon(Application::iconManager()->iconForUrl(u));
+}
