@@ -83,6 +83,7 @@ public:
         KGet    = 0x00000001,
         RSS     = 0x00000010,
         SSL     = 0x00000100,
+        BK      = 0x00001000
     };
 
     explicit UrlBar(QWidget *parent = 0);
@@ -104,6 +105,11 @@ private slots:
     void detectTypedString(const QString &);
     void suggest();
 
+    void showBookmarkInfo(const QPoint &pos);
+    void onBookmarksChanged();
+
+    void refreshFavicon();
+    
 protected:
     void paintEvent(QPaintEvent *event);
     void keyPressEvent(QKeyEvent *event);
