@@ -32,18 +32,18 @@
 #include "rekonq.h"
 
 // KDE Includes
-#include <klocalizedstring.h>
 #include <KIcon>
+#include <KLocalizedString>
 
 // Qt Includes
-#include <QToolButton>
-#include <QPushButton>
-#include <QHBoxLayout>
-#include <QString>
+#include <QtGui/QGridLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+#include <QtGui/QToolButton>
 
 
 WalletBar::WalletBar(QWidget *parent)
-        : QWidget(parent)
+        : NotificationBar(parent)
         , m_label(new QLabel(this))
 {
     m_label->setWordWrap(true);
