@@ -38,19 +38,11 @@
 
 // Local Includes
 #include "protocolhandler.h"
-#include "newtabpage.h"
 #include "websslinfo.h"
 
 // KDE Includes
 #include <KWebPage>
 
-// Qt Includes
-#include <QUrl>
-
-// Forward Declarations
-class QWebFrame;
-class QNetworkReply;
-
 
 class REKONQ_TESTS_EXPORT WebPage : public KWebPage
 {
@@ -66,12 +58,12 @@ public:
     inline bool isOnRekonqPage() const { return _isOnRekonqPage; };
     inline void setIsOnRekonqPage(bool b) { _isOnRekonqPage = b; };
 
-public slots:
+public Q_SLOTS:
     void downloadAllContentsWithKGet(QPoint);
-    
+
     virtual void downloadRequest(const QNetworkRequest &request);
     virtual void downloadUrl(const KUrl &url);
-    
+
 protected:
     WebPage *createWindow(WebWindowType type);
 
@@ -79,22 +71,23 @@ protected:
                                          const QNetworkRequest &request,
                                          NavigationType type);
 
-private slots:
+private Q_SLOTS:
     void handleUnsupportedContent(QNetworkReply *reply);
     void manageNetworkErrors(QNetworkReply *reply);
+    void loadStarted();
     void loadFinished(bool);
     void showSSLInfo(QPoint);
     void updateImage(bool ok);
 
     void downloadReply(const QNetworkReply *reply, const QString &suggestedFileName = QString());
-    
-private:  
+
+private:
     QString errorPage(QNetworkReply *reply);
     QUrl _loadingUrl;
 
     ProtocolHandler _protHandler;
     WebSslInfo _sslInfo;
-    
+
     bool _networkAnalyzer;
     bool _isOnRekonqPage;
 };
