@@ -31,8 +31,7 @@
 
 // Local Includes
 #include "rekonq.h"
-#include "application.h"
-#include "mainwindow.h"
+
 #include "clicktoflash.h"
 
 
