@@ -38,7 +38,6 @@
 
 // Qt Includes
 #include <QtCore/QSize>
-#include <QtCore/QTimer>
 #include <QtCore/QFile>
 
 #include <QtGui/QPainter>
@@ -61,7 +60,7 @@ WebSnap::WebSnap(const KUrl& url, QObject *parent)
 
     connect(&m_page, SIGNAL(loadFinished(bool)), this, SLOT(saveResult(bool)));
 
-    QTimer::singleShot(0, this, SLOT(load()));
+    QMetaObject::invokeMethod(this, "load", Qt::QueuedConnection);
 }
 
 
@@ -184,20 +183,9 @@ QPixmap WebSnap::renderPagePreview(const QWebPage &page, int w, int h)
 QString WebSnap::imagePathFromUrl(const KUrl &url)
 {
     QUrl temp = QUrl(url.url());
-    QString name = temp.toString(QUrl::RemoveScheme | QUrl::RemoveUserInfo | QUrl::StripTrailingSlash);
+    QByteArray name = temp.toEncoded(QUrl::RemoveScheme | QUrl::RemoveUserInfo | QUrl::StripTrailingSlash);
 
-    // TODO learn Regular Expressions :)
-    // and implement something better here..
-    name.remove('/');
-    name.remove('&');
-    name.remove('.');
-    name.remove('-');
-    name.remove('_');
-    name.remove('?');
-    name.remove('=');
-    name.remove('+');
-
-    return KStandardDirs::locateLocal("cache", QString("thumbs/") + name + ".png", true);
+    return KStandardDirs::locateLocal("cache", QString("thumbs/") + name.toBase64() + ".png", true);
 }
 
 
@@ -205,7 +193,7 @@ void WebSnap::saveResult(bool ok)
 {
     if (ok)
     {
-        QPixmap image = renderPagePreview(m_page, WIDTH, HEIGHT);
+        QPixmap image = renderPagePreview(m_page, defaultWidth, defaultHeight);
         QString path = imagePathFromUrl(m_url);
         QFile::remove(path);
         image.save(path);
