@@ -38,15 +38,10 @@
 
 // Qt Includes
 #include <QtCore/QObject>
-
-#include <QtGui/QPixmap>
-
 #include <QtWebKit/QWebPage>
 
-// Defines
-#define WIDTH  200
-#define HEIGHT 150
-
+// Forward Declarations
+class QPixmap;
 
 /**
  * This class is used in many classes of rekonq to produce an image
@@ -63,6 +58,7 @@
  * - NewTabPage class:      to show the favorites page "preview" (given an url, you show AND save an image)
  *
  */
+
 class REKONQ_TESTS_EXPORT WebSnap : public QObject
 {
     Q_OBJECT
@@ -78,7 +74,7 @@ public:
     explicit WebSnap(const KUrl &url, QObject *parent = 0);
 
     ~WebSnap();
-    
+
     /**
      * Snaps a pixmap of size w * h from a page
      *
@@ -88,10 +84,10 @@ public:
      *
      * @return the pixmap snapped from the page
      */
-    static QPixmap renderPagePreview(const QWebPage &page, int w = WIDTH, int h = HEIGHT);
-        
+    static QPixmap renderPagePreview(const QWebPage &page, int w = defaultWidth, int h = defaultHeight);
+
     // static QPixmap renderVisiblePagePreview(const QWebPage &page, int w = WIDTH, int h = HEIGHT); TODO: try to make this method work => more previews for the urlbar
-    
+
     /**
      * Snaps a pixmap of size w * h from a page of a tab that is currently closing
      *
@@ -101,8 +97,8 @@ public:
      *
      * @return the pixmap snapped from the page
      */
-    static QPixmap renderClosingPagePreview(const QWebPage &page, int w = WIDTH, int h = HEIGHT);
-    
+    static QPixmap renderClosingPagePreview(const QWebPage &page, int w = defaultWidth, int h = defaultHeight);
+
     /**
      * Snaps a pixmap of size w * h from a page for tab preview
      *
@@ -112,8 +108,8 @@ public:
      *
      * @return the pixmap snapped from the page
      */
-    static QPixmap renderTabPreview(const QWebPage &page, int w, int h);   
-    
+    static QPixmap renderTabPreview(const QWebPage &page, int w, int h);
+
     /**
      * Guess the local path where the image for the url provided
      * should be
@@ -139,9 +135,12 @@ signals:
     void snapDone(bool ok);
 
 private:
+    // Constants
+    static const int defaultWidth = 200;
+    static const int defaultHeight = 150;
     QWebPage m_page;
     KUrl m_url;
-    
+
     //render a preview: common part of renderPagePreview() and renderTabPreview()
     static QPixmap render(const QWebPage &page, int w, int h);
 };
