@@ -33,30 +33,15 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "application.h"
-#include "mainwindow.h"
-#include "mainview.h"
-#include "bookmarksmanager.h"
-#include "walletbar.h"
 #include "previewselectorbar.h"
 #include "rsswidget.h"
+#include "walletbar.h"
+#include "webpage.h"
 
 // KDE Includes
-#include <KService>
-#include <KUriFilterData>
-#include <KStandardShortcut>
-#include <KMenu>
-#include <KActionMenu>
-#include <KWebView>
-#include <kwebwallet.h>
-#include <KDE/KMessageBox>
+#include <KWebWallet>
 
 // Qt Includes
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QWheelEvent>
-#include <QtGui/QMouseEvent>
-#include <QtGui/QClipboard>
-#include <QtGui/QKeyEvent>
 #include <QtGui/QVBoxLayout>
 
 
@@ -66,7 +51,7 @@ WebTab::WebTab(QWidget *parent)
         , m_progress(0)
 {
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-    
+
     QVBoxLayout *l = new QVBoxLayout(this);
     l->setMargin(0);
     l->setSpacing(0);
@@ -92,6 +77,8 @@ WebTab::WebTab(QWidget *parent)
 
 WebTab::~WebTab()
 {
+    _walletBar.clear();
+    _previewSelectorBar.clear();
 }
 
 
@@ -143,26 +130,37 @@ void WebTab::createWalletBar(const QString &key, const QUrl &url)
         return;
 
     KWebWallet *wallet = page()->wallet();
-    WalletBar *walletBar = new WalletBar(this);
-    walletBar->onSaveFormData(key, url);
-    qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, walletBar);
+    if(_walletBar.isNull()) {
+        _walletBar = new WalletBar(this);
+        _walletBar.data()->onSaveFormData(key, url);
+        qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, _walletBar.data() );
+    } else {
+        disconnect(wallet);
+        _walletBar.data()->notifyUser();
+    }
 
-    connect(walletBar, SIGNAL(saveFormDataAccepted(const QString &)),
-            wallet, SLOT(acceptSaveFormDataRequest(const QString &)));
-    connect(walletBar, SIGNAL(saveFormDataRejected(const QString &)),
-            wallet, SLOT(rejectSaveFormDataRequest(const QString &)));
+    connect(_walletBar.data(), SIGNAL(saveFormDataAccepted(const QString &)),
+            wallet, SLOT(acceptSaveFormDataRequest(const QString &)), Qt::UniqueConnection);
+    connect(_walletBar.data(), SIGNAL(saveFormDataRejected(const QString &)),
+            wallet, SLOT(rejectSaveFormDataRequest(const QString &)), Qt::UniqueConnection);
 }
 
 
 void WebTab::createPreviewSelectorBar(int index)
 {
-    PreviewSelectorBar *bar = new PreviewSelectorBar(index, this);
-    qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, bar);
-    
-    connect(page(), SIGNAL(loadStarted()), bar, SLOT(loadProgress()));
-    connect(page(), SIGNAL(loadProgress(int)), bar, SLOT(loadProgress()));
-    connect(page(), SIGNAL(loadFinished(bool)), bar, SLOT(loadFinished()));
-    connect(page()->mainFrame(), SIGNAL(urlChanged(QUrl)), bar, SLOT(verifyUrl()));
+    if(_previewSelectorBar.isNull()) {
+        _previewSelectorBar = new PreviewSelectorBar(index, this);
+        qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, _previewSelectorBar.data());
+    } else {
+        disconnect(_previewSelectorBar.data());
+        _previewSelectorBar.data()->setIndex(index);
+        _previewSelectorBar.data()->notifyUser();
+    }
+
+    connect(page(),             SIGNAL(loadStarted()),      _previewSelectorBar.data(), SLOT(loadProgress()), Qt::UniqueConnection);
+    connect(page(),             SIGNAL(loadProgress(int)),  _previewSelectorBar.data(), SLOT(loadProgress()), Qt::UniqueConnection);
+    connect(page(),             SIGNAL(loadFinished(bool)), _previewSelectorBar.data(), SLOT(loadFinished()), Qt::UniqueConnection);
+    connect(page()->mainFrame(), SIGNAL(urlChanged(QUrl)),  _previewSelectorBar.data(), SLOT(verifyUrl()), Qt::UniqueConnection);
 }
 
 
@@ -183,7 +181,7 @@ void WebTab::showRSSInfo(QPoint pos)
     col.append(page()->mainFrame()->findAllElements("link[type=\"application/atom+xml\"]"));
 
     QMap<KUrl, QString> map;
-    
+
     foreach(const QWebElement &el, col)
     {
         QString urlString;
