@@ -33,51 +33,52 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "mainwindow.h"
+#include "application.h"
+#include "bookmarkprovider.h"
+#include "iconmanager.h"
 #include "mainview.h"
-#include "webpage.h"
-#include "bookmarksmanager.h"
+#include "mainwindow.h"
 #include "searchengine.h"
+#include "urlbar.h"
+#include "webpage.h"
 #include "websnap.h"
 
 // KDE Includes
-#include <KService>
-#include <KUriFilterData>
-#include <KStandardShortcut>
-#include <KMenu>
+#include <KAction>
 #include <KActionMenu>
-#include <ktoolinvocation.h>
+#include <KLocalizedString>
+#include <KMenu>
+#include <KStandardAction>
+#include <KStandardDirs>
 
 // Qt Includes
-#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtCore/QTimer>
 
-#include <QtGui/QAction>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QWheelEvent>
-#include <QtGui/QMouseEvent>
 #include <QtGui/QClipboard>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QLayout>
+#include <QtGui/QContextMenuEvent>
 #include <QtGui/QWindowsStyle>
 
-#include <QtDBus/QDBusConnectionInterface>
-#include <QtDBus/QDBusInterface>
-#include <QtDBus/QDBusReply>
+#include <QtWebKit/QWebFrame>
+#include <QtWebKit/QWebHistory>
 
 
 WebView::WebView(QWidget* parent)
         : KWebView(parent, false)
         , _mousePos(QPoint(0, 0))
-        , _scrollTimer(new QTimer(this))
+        , _autoScrollTimer(new QTimer(this))
         , _VScrollSpeed(0)
         , _HScrollSpeed(0)
         , _canEnableAutoScroll(true)
         , _isAutoScrollEnabled(false)
+        , _smoothScrollTimer(new QTimer(this))
+        , _smoothScrolling(false)
+        , _dy(0)
 {
     WebPage *page = new WebPage(this);
     setPage(page);
-    
-    // // NOTE This is a lot hackish. We copied it from Arora, but using a "Windows Style" 
+
+    // // NOTE This is a lot hackish. We copied it from Arora, but using a "Windows Style"
     // // seems really a pity to me. The problem is that using a KStyle everything seems "broken"
     // // (at least with dark colors). So I think the code should be somthing like:
     // KStyle s;
@@ -90,7 +91,7 @@ WebView::WebView(QWidget* parent)
         p = s.standardPalette();
         setPalette(p);
     }
-    
+
     // download system
     connect(this, SIGNAL(linkShiftClicked(const KUrl &)), page, SLOT(downloadUrl(const KUrl &)));
     connect(page, SIGNAL(downloadRequested(const QNetworkRequest &)), page, SLOT(downloadRequest(const QNetworkRequest &)));
@@ -102,19 +103,28 @@ WebView::WebView(QWidget* parent)
     connect(this, SIGNAL(loadUrl(const KUrl &, const Rekonq::OpenType &)),
             Application::instance(), SLOT(loadUrl(const KUrl &, const Rekonq::OpenType &)));
 
-    // scrolling timer
-    connect(_scrollTimer, SIGNAL(timeout()), this, SLOT(scrollFrameChanged()));
-    _scrollTimer->setInterval(100);
+    // Auto scroll timer
+    connect(_autoScrollTimer, SIGNAL(timeout()), this, SLOT(scrollFrameChanged()));
+    _autoScrollTimer->setInterval(100);
+
+    // Smooth scroll timer
+    connect(_smoothScrollTimer, SIGNAL(timeout()), this, SLOT(scrollTick()));
+    _smoothScrollTimer->setInterval(16);
 }
 
 
 WebView::~WebView()
 {
-    delete _scrollTimer;
+    delete _autoScrollTimer;
+
+    if (_smoothScrolling)
+        stopScrolling();
+    delete _smoothScrollTimer;
+
     disconnect();
 
     WebPage* p = page();
-    
+
     QPixmap preview = WebSnap::renderClosingPagePreview(*p);
     QString path = WebSnap::imagePathFromUrl(p->mainFrame()->url().toString());
     QFile::remove(path);
@@ -156,7 +166,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
         menu.addAction(pageAction(KWebPage::DownloadLinkToDisk));
         menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
-        menu.addSeparator();        
+        menu.addSeparator();
     }
 
     // is content editable && selected? Add CUT
@@ -191,7 +201,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         foreach(KService::Ptr engine, SearchEngine::favorites())
         {
             a = new KAction(engine->name(), this);
-            a->setIcon(Application::icon(SearchEngine::buildQuery(engine, "")));
+            a->setIcon(Application::iconManager()->iconForUrl(SearchEngine::buildQuery(engine, "")));
             a->setData(engine->entryPath());
             connect(a, SIGNAL(triggered(bool)), this, SLOT(search()));
             searchMenu->addAction(a);
@@ -203,7 +213,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         }
 
         menu.addSeparator();
-        
+
         menu.addAction(inspectAction);
         // TODO Add translate, show translation
     }
@@ -221,8 +231,15 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
         menu.addAction(pageAction(KWebPage::DownloadImageToDisk));
         menu.addAction(pageAction(KWebPage::CopyImageToClipboard));
+
+        a = new KAction(KIcon("view-media-visualization"), i18n("&Copy Image Location"), this);
+        a->setData(result.imageUrl());
+        connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(slotCopyImageLocation()));
+        menu.addAction(a);
+
+        menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
         menu.addSeparator();
-        
+
         menu.addAction(inspectAction);
     }
 
@@ -305,7 +322,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
             menu.addAction(mainwindow->actionByName(KStandardAction::name(KStandardAction::SaveAs)));
 
-            if (ReKonfig::kgetList())
+            if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
             {
                 a = new KAction(KIcon("kget"), i18n("List All Links"), this);
                 connect(a, SIGNAL(triggered(bool)), page(), SLOT(downloadAllContentsWithKGet()));
@@ -338,7 +355,7 @@ void WebView::mousePressEvent(QMouseEvent *event)
         setCursor(Qt::ArrowCursor);
         _VScrollSpeed = 0;
         _HScrollSpeed = 0;
-        _scrollTimer->stop();
+        _autoScrollTimer->stop();
         _isAutoScrollEnabled = false;
         return;
     }
@@ -381,21 +398,23 @@ void WebView::mouseMoveEvent(QMouseEvent *event)
         QPoint r = _mousePos - _clickPos;
         _HScrollSpeed = r.x() / 2;  // you are too fast..
         _VScrollSpeed = r.y() / 2;
-        if (!_scrollTimer->isActive())
-            _scrollTimer->start();
+        if (!_autoScrollTimer->isActive())
+            _autoScrollTimer->start();
 
         return;
     }
 
-    if (Application::instance()->mainWindow()->isFullScreen())
+    MainWindow *w = Application::instance()->mainWindow();
+    if (w->isFullScreen())
     {
         if (event->pos().y() >= 0 && event->pos().y() <= 4)
         {
-            Application::instance()->mainWindow()->setWidgetsVisible(true);
+            w->setWidgetsVisible(true);
         }
         else
         {
-            Application::instance()->mainWindow()->setWidgetsVisible(false);
+            if(!w->mainView()->urlBar()->hasFocus())
+                w->setWidgetsVisible(false);
         }
     }
     KWebView::mouseMoveEvent(event);
@@ -408,7 +427,7 @@ void WebView::search()
     KService::Ptr engine = KService::serviceByDesktopPath(a->data().toString());
     KUrl urlSearch = KUrl(SearchEngine::buildQuery(engine, selectedText()));
 
-    emit loadUrl(urlSearch, Rekonq::NewFocusedTab);
+    emit loadUrl(urlSearch, Rekonq::NewTab);
 }
 
 
@@ -433,6 +452,23 @@ void WebView::viewImage(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifier
     }
 }
 
+void WebView::slotCopyImageLocation()
+{
+    KAction *a = qobject_cast<KAction*>(sender());
+    KUrl imageUrl (a->data().toUrl());
+#ifndef QT_NO_MIMECLIPBOARD
+    // Set it in both the mouse selection and in the clipboard
+    QMimeData* mimeData = new QMimeData;
+    imageUrl.populateMimeData( mimeData );
+    QApplication::clipboard()->setMimeData( mimeData, QClipboard::Clipboard );
+    mimeData = new QMimeData;
+    imageUrl.populateMimeData( mimeData );
+    QApplication::clipboard()->setMimeData( mimeData, QClipboard::Selection );
+#else
+    QApplication::clipboard()->setText( imageUrl.url() );
+#endif
+}
+
 
 void WebView::openLinkInNewWindow()
 {
@@ -481,43 +517,43 @@ void WebView::keyPressEvent(QKeyEvent *event)
         if (event->key() == Qt::Key_Up)
         {
             _VScrollSpeed--;
-            if (!_scrollTimer->isActive())
-                _scrollTimer->start();
+            if (!_autoScrollTimer->isActive())
+                _autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Down)
         {
             _VScrollSpeed++;
-            if (!_scrollTimer->isActive())
-                _scrollTimer->start();
+            if (!_autoScrollTimer->isActive())
+                _autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Right)
         {
             _HScrollSpeed++;
-            if (!_scrollTimer->isActive())
-                _scrollTimer->start();
+            if (!_autoScrollTimer->isActive())
+                _autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Left)
         {
             _HScrollSpeed--;
-            if (!_scrollTimer->isActive())
-                _scrollTimer->start();
+            if (!_autoScrollTimer->isActive())
+                _autoScrollTimer->start();
             return;
         }
 
-        if (_scrollTimer->isActive())
+        if (_autoScrollTimer->isActive())
         {
-            _scrollTimer->stop();
+            _autoScrollTimer->stop();
         }
         else
         {
             if (_VScrollSpeed || _HScrollSpeed)
-                _scrollTimer->start();
+                _autoScrollTimer->start();
         }
     }
 
@@ -527,7 +563,10 @@ void WebView::keyPressEvent(QKeyEvent *event)
 
 void WebView::wheelEvent(QWheelEvent *event)
 {
+    // To let some websites (eg: google maps) to handle wheel events
+    int prevPos = page()->currentFrame()->scrollPosition().y();
     KWebView::wheelEvent(event);
+    int newPos = page()->currentFrame()->scrollPosition().y();
 
     // Sync with the zoom slider
     if (event->modifiers() == Qt::ControlModifier)
@@ -543,7 +582,23 @@ void WebView::wheelEvent(QWheelEvent *event)
         if ((zoomFactor() * 10 - newFactor) > 0.5)
             newFactor++;
 
-        emit zoomChanged((qreal)newFactor / 10);
+        emit zoomChanged(newFactor);
+    }
+    else if (ReKonfig::smoothScrolling() && prevPos != newPos)
+    {
+
+        page()->currentFrame()->setScrollPosition(QPoint(page()->currentFrame()->scrollPosition().x(), prevPos));
+
+        if ((event->delta() > 0) != !_scrollBottom)
+            stopScrolling();
+
+        if (event->delta() > 0)
+            _scrollBottom = false;
+        else
+            _scrollBottom = true;
+
+
+        setupSmoothScrolling(abs(newPos - prevPos));
     }
 }
 
@@ -577,3 +632,84 @@ void WebView::scrollFrameChanged()
     if (x == 0 || x == page()->currentFrame()->scrollBarMaximum(Qt::Horizontal))
         _HScrollSpeed = 0;
 }
+
+
+void WebView::setupSmoothScrolling(int posY)
+{
+    int ddy = qMax(_smoothScrollSteps ? abs(_dy)/_smoothScrollSteps : 0,3);
+
+    _dy += posY;
+
+    if (_dy <= 0)
+    {
+        stopScrolling();
+        return;
+    }
+
+    _smoothScrollSteps = 8;
+
+    if (_dy / _smoothScrollSteps < ddy)
+    {
+        _smoothScrollSteps = (abs(_dy)+ddy-1)/ddy;
+        if (_smoothScrollSteps < 1)
+            _smoothScrollSteps = 1;
+    }
+
+    _smoothScrollTime.start();
+
+    if (!_smoothScrolling)
+    {
+        _smoothScrolling = true;
+        _smoothScrollTimer->start();
+        scrollTick();
+    }
+}
+
+
+void WebView::scrollTick()
+{
+    if (_dy == 0)
+    {
+        stopScrolling();
+        return;
+    }
+
+    if (_smoothScrollSteps < 1)
+        _smoothScrollSteps = 1;
+
+    int takesteps = _smoothScrollTime.restart() / 16;
+    int scroll_y = 0;
+
+    if (takesteps < 1)
+        takesteps = 1;
+
+    if (takesteps > _smoothScrollSteps)
+        takesteps = _smoothScrollSteps;
+
+    for(int i = 0; i < takesteps; i++)
+    {
+        int ddy = (_dy / (_smoothScrollSteps+1)) * 2;
+
+        // limit step to requested scrolling distance
+        if (abs(ddy) > abs(_dy))
+            ddy = _dy;
+
+        // update remaining scroll
+        _dy -= ddy;
+        scroll_y += ddy;
+        _smoothScrollSteps--;
+    }
+
+    if (_scrollBottom)
+        page()->currentFrame()->scroll(0, scroll_y);
+    else
+        page()->currentFrame()->scroll(0, -scroll_y);
+}
+
+
+void WebView::stopScrolling()
+{
+    _smoothScrollTimer->stop();
+    _dy = 0;
+    _smoothScrolling = false;
+}
