@@ -35,6 +35,9 @@
 // KDE Includes
 #include <KWebView>
 
+//Qt Includes
+#include <QtCore/QTime>
+
 // Forward Declarations
 class WebPage;
 
@@ -48,7 +51,7 @@ public:
     ~WebView();
 
     WebPage *page();
-    
+
     inline QPoint mousePos() { return _mousePos; }
 
 protected:
@@ -68,23 +71,38 @@ private slots:
     void openLinkInNewTab();
 
     void viewImage(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);
+    void slotCopyImageLocation();
     void inspect();
 
     void scrollFrameChanged();
+    void scrollTick();
+
+    void setupSmoothScrolling(int posY);
+
+    void stopScrolling();
 
 signals:
     void loadUrl(const KUrl &, const Rekonq::OpenType &);
-    void zoomChanged(qreal);
+    void zoomChanged(int);
 
 private:
     QPoint _mousePos;
     QPoint _clickPos;
 
-    QTimer *_scrollTimer;
+    // Auto Scroll
+    QTimer *_autoScrollTimer;
     int _VScrollSpeed;
     int _HScrollSpeed;
     bool _canEnableAutoScroll;
     bool _isAutoScrollEnabled;
+
+    // Smooth Scroll
+    QTimer *_smoothScrollTimer;
+    QTime _smoothScrollTime;
+    bool _scrollBottom;
+    bool _smoothScrolling;
+    int _dy;
+    int _smoothScrollSteps;
 };
 
 #endif
